/*
  mexxmorphwarp    MEX file source

  Given two meshes (source and destination), read by readxmorphmesh
  (comming from xmorph), calculate the final position for each source
  pixel. Both meshes must have the same s

  Calling: [xw,yw]=mexxmorphwarp(xs,ys,xd,yd,imw,imh) ;

  Attention: Image coordinates (for xs..yd) are taken to start by zero
  and go up to imw-1 resp. imh-1, and the border points *must* lie there
  for both grids! If you get a lot of 'hermite3_array: out of range',
  verify this.

  Jan Kybic, November 1998

*/

#define myCreateMatrix(a,b) mxCreateDoubleMatrix(a,b,mxREAL)
#define myAlloc(a) mxMalloc(a)


#include <string.h>
#include <math.h>
#include "mex.h"

/* Input Arguments */

#define	XS_IN	prhs[0]
#define	YS_IN	prhs[1]
#define	XD_IN	prhs[2]
#define YD_IN   prhs[3]
#define IMW_IN  prhs[4]
#define IMH_IN  prhs[5]

/* Output Arguments */

#define	XW_OUT	plhs[0]
#define	YW_OUT	plhs[1]

#define NAME "MEXXMORPHWARP"

void
xmorphwarp(int img_width, int img_height,
  const double *xs, const double *ys, const double *xd, const double *yd,
  int mesh_width, int mesh_height,double *warpx, double *warpy) ;

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *xs,*ys,*xd,*yd,*warpx,*warpy ;
  int img_width,img_height,mesh_width,mesh_height ;
  int i ;

  /* Check for proper number of arguments */
  
  if (nrhs != 6) {
    mexErrMsgTxt(NAME " requires six input arguments.");
  } else if (nlhs != 2) {
    mexErrMsgTxt(NAME " requires two output argument.");
  }
  
  for (i=0;i<nrhs;i++)
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(
             NAME " needs all input arguments to be real.");
     } 

  /* Matlab stores arrays transposed with respect to C code */
  /* we just ignore this because of the nature of our problem */
  /* Nevertheless, we have to get the size parameters right */
  /* We have to swap x and y and width and height, too */

  mesh_width=mxGetM(XS_IN) ; mesh_height=mxGetN(XS_IN) ;

  xs=mxGetPr(XS_IN) ; ys=mxGetPr(YS_IN) ;
  xd=mxGetPr(XD_IN) ; yd=mxGetPr(YD_IN) ;

  img_width=(int)(*(mxGetPr(IMH_IN))) ;
  img_height=(int)(*(mxGetPr(IMW_IN))) ;

  YW_OUT = myCreateMatrix(img_width,img_height);
  XW_OUT = myCreateMatrix(img_width,img_height);
  warpx=mxGetPr(XW_OUT) ;
  warpy=mxGetPr(YW_OUT) ;

#if 0
  mexPrintf("calling xmorphwarp with img_width=%d, img_height=%d, mesh_width=%d"
	    " mesh_height=%d\n",img_width,img_height, mesh_width, mesh_height) ;
  mexPrintf("xs[0]=%g ys[0]=%g xd[0]=%d yd[0]=%d\n",*xs,*ys,*xd,*yd) ; 
#endif
  xmorphwarp(img_width,img_height,ys,xs,yd,xd,mesh_width,mesh_height,
	     warpy,warpx) ;
}

#ifndef FALSE
#define FALSE 0
#endif
#define MY_CALLOC(s,t) (void *)mxMalloc(s*sizeof(t))
#define FREE(x) mxFree(x)
#define printf  mexPrintf

#define MAKINGMEX

#include "xmorphwarp.c"
