/* To compile with 'cmex ddemo.c -ldl' */
#include <dlfcn.h>

#include "mex.h"

/* BEGIN */
void determine_class(mxArray  *array_ptr)
{
 mxClassID  category;
 /* Display the class name of each input mxArray. */ 
   category = mxGetClassID(array_ptr);
   switch (category) {
      case mxCHAR_CLASS:    mexPrintf("String ");       break;
      case mxSTRUCT_CLASS:  mexPrintf("Structure ");    break;
      case mxSPARSE_CLASS:  mexPrintf("Sparse ");       break; 
      case mxCELL_CLASS:    mexPrintf("Cell ");         break; 
      case mxUNKNOWN_CLASS: mexWarnMsgTxt("Unknown Class."); break;
      default:              mexPrintf("Full Numeric "); break; 
   } 
}

void
mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
 mxArray   *array_ptr, *new_array_ptr;
 int        c;
 double     *pr ; 
 float *ptr;
 int dims[3]={2,2,2} ;
 int ndim ; const int *dimsd ;
 char buf[33] ;
 void *handle ;
 int (*func)() ;

 /* Examine input (right-hand-side) arguments. */
   mexPrintf("\nThere are %d right-hand-side arguments.", nrhs);
   for (c=0; c<nrhs; c++)  {
      mexPrintf("\n\tInput Arg %d: ", c+1);
      array_ptr = (mxArray *)prhs[c];
      determine_class(array_ptr);
   }

   /* Take the first argument */
   pr=mxGetPr(prhs[0]) ;
   mexPrintf("\nValue of the first argument: %g\n",*pr) ;

   /* Change the value of the first parameter */
   *pr=4 ;

   /* The third parameter is a string */
   mxGetString(prhs[2],buf,32);
   mexPrintf("Value of the third parameter: %s\n",buf) ;

   /* The fourth argument is a multidimensional array */
   ndim=mxGetNumberOfDimensions(prhs[3]) ;
   dimsd=mxGetDimensions(prhs[3]);
   mexPrintf("Fourth argument has %d dimensions.\n",ndim) ;

   /* Evaluate some Matlab expressions */
   mexEvalString("e=34*7") ;

   /* Examine output (left-hand-side) arguments. */
   mexPrintf("\n\nThere are %d left-hand-side arguments.\n", nlhs);

   /* return random number as the first left argument */
   new_array_ptr = mxCreateDoubleMatrix(0,0,mxCOMPLEX);
   pr=(double *)mxCalloc(1,mxGetElementSize(new_array_ptr));
   *pr = rand();

   /* set size */
   mxSetM(new_array_ptr, 1);
   mxSetN(new_array_ptr, 1);
   mxSetPr(new_array_ptr, pr);
   mxSetPi(new_array_ptr, NULL);
   /* Put a random scalar into the left-hand-side argument. */
   plhs[0] = new_array_ptr; 

   /* Create 3D matrix and return it as the second argument */
   
   if ((plhs[1]=mxCreateNumericArray(3,dims,mxSINGLE_CLASS,mxREAL))
       ==NULL) mexErrMsgTxt("Could not create array.") ;
   ptr=(float *)mxGetPr(plhs[1]) ;

   /* Dynamic linking */
   handle=dlopen("./dynamic.so",RTLD_NOW | RTLD_LOCAL) ;
   if (handle==NULL) mexErrMsgTxt(dlerror()) ;
   func=(int (*)(void))dlsym(handle,"dynamic") ;
   /* Invoke function from the dynamic library and return result 
      in the first element of the second output parameter */
   mexPrintf("Dynamically called function returned %d\n",(*func)()) ;
   *ptr=(*func)() ;
   /* Deallocate dynamic library */
   dlclose(handle) ;
}
/* END */



