function fig = disptri(VV,d)
% Displays 2D projections (cuts) of 3D volume VV
% While permitting user to navigate around the volume by mouse-clicking directly
% on the projections or moving sliders
%
% d=[dx dy dz] gives sizes of voxels in respective dimensions
%
% My first project on Matlab GUI, therefore far from perfect.
% Jan Kybic, October 1998

global V h1 h2 h3 c a1 a2 a3 dims;

if nargin<2, d=[1 1 1] ; end ;

% This is a trick to put all the callback routines in one file
if (isstr(VV)),
  if (strcmp(VV,'slider')),
    disptrislider ;
  elseif (strcmp(VV,'show')),
    disptrishow ;
  elseif (strcmp(VV,'mouse')),
    disptrimouse ;
  end ;
else

dims=d ;
% size of the window
x1=1000 ; y1=800 ;
ew=70 ; % element width
eh=20 ; % element height
es=25 ; % element step

maxcol=63 ; % colormap size
V=VV-min(min(min(VV))) ;
V=V/max(max(max(V)))*maxcol+1 ;

po='Position' ; pr='Parent' ;

% Populate window
h0=gcf ; clf ; zoom off ;
hp=get(h0,po) ;
% x1=hp(3) ; y1=hp(4) ;
hp(3)=x1 ; hp(4)=y1 ;
set(h0,po,hp) ;
set(gcf,'resize','off') ;

xo=x1/2+20 ; yo=y1/2-10 ;
subplot(221) ; fgr=gcf ; a1=gca ;
subplot(222) ; a2=gca ; 
subplot(223) ; a3=gca ;
h1 = uicontrol(pr,h0,po,[xo yo-es ew eh],'Style','Slider','Tag','Slider1');
h2 = uicontrol(pr,h0,po,[xo yo-2*es ew eh],'Style','Slider','Tag','Slider2');
h3 = uicontrol(pr,h0,po,[xo yo-3*es ew eh],'Style','Slider','Tag','Slider3');
h4 = uicontrol(pr,h0,po,[xo yo-4*es ew eh],'String','Quit','Tag','Pushbutton1');

sv=size(V) ; c=round(sv/2) ;

set(h1,'Min',1) ; set(h1,'Max',sv(1)) ; 
set(h2,'Min',1) ; set(h2,'Max',sv(2)) ;
set(h3,'Min',1) ; set(h3,'Max',sv(3)) ;
set(h1,'Callback','disptri(''slider'')') ;
set(h2,'Callback','disptri(''slider'')') ;
set(h3,'Callback','disptri(''slider'')') ;
set(h4,'Callback','clf ; return ;') ;

set(fgr,'WindowButtonUpFcn','disptri(''mouse'')') ;

disptri('show');

end ;

%----------------------------------------------------------------------

function disptrislider ;
% Callback for disptri
global V h1 h2 h3 c ;
c(1)=round(get(h1,'Value')) ;
c(2)=round(get(h2,'Value')) ;
c(3)=round(get(h3,'Value')) ;
disptrishow ;

%----------------------------------------------------------------------

function disptrishow ;
% helper function for disptri
% show V cut at c
global V h1 h2 h3 c dims;
sv=size(V) ;
colormap(bone) ;
subplot(221) ; image(V(:,:,c(3))') ; axis image ; axis off ;
set(gca,'DataAspectRatio',[ dims(2) dims(1) 1 ]) ;

hold on ; plot([1 sv(1)],[c(2) c(2)],'r-') ;
plot([c(1) c(1)],[1 sv(2)],'r-') ; hold off ;

subplot(222) ; image(reshape(V(c(1),:,:),sv(2),sv(3))) ; axis image ; axis off ;
set(gca,'DataAspectRatio',[ dims(2) dims(3) 1 ]) ;

hold on ; plot([1 sv(3)],[c(2) c(2)],'r-') ;
plot([c(3) c(3)],[1 sv(2)],'r-') ; hold off ;

subplot(223) ; image(reshape(V(:,c(2),:),sv(1),sv(3))'); axis image ; axis off; 
set(gca,'DataAspectRatio',[ dims(3) dims(1) 1 ]) ;
hold on ; plot([1 sv(1)],[c(3) c(3)],'r-') ;
plot([c(1) c(1)],[1 sv(3)],'r-') ; hold off ;

set(h1,'Value',c(1)) ;
set(h2,'Value',c(2)) ;
set(h3,'Value',c(3)) ;

%----------------------------------------------------------------------

function disptrimouse
global a1 a2 a3 c;

a=gca ;
x=get(a,'CurrentPoint') ;
x=round(x) ; 
x=x(1,1:2) ;
if a==a1, c=[x c(3)] ; end ;
if a==a2, c=[c(1) x(2) x(1)] ; end ;
if a==a3, c=[x(1) c(2) x(2)] ; end ;
disptrishow ;
