function [x,t]=fourtotime(fnc,sup,T) ;
% Given a function fnc (given as string) of a Fourier domain function
% correcponding to an effectively finite support function (with support
% -sup..sup), calculates fnc in time domain.  T point FFT is used.
%
% Example: [x,t]=fourtotime('(o>0)*sin(o/2)/(o/2+(o==0))+(o==0)',0.5) ;
% displays a spline of degree 0

if nargin<3,  T=1024 ;       % FFT length
end ;

unit=T/8/sup ; % this many samples correspond to unit time
scale=2*pi*unit/T ;
omg=(0:T/2) ;
somg=omg*scale ;
f=zeros(size(omg)) ;

for i=1:T/2, 
  o=somg(i) ;
  eval(['f(i)=' fnc ' ;']) ;
end ;

f=[ f conj(f(T/2:-1:2)) ]*unit ;
x=real(ifft(f)) ;
x=[ x(T/2+1:T) x(1:T/2) ] ;
t=[-T/2:T/2-1]/unit ;

