function fig = disptritwo1(VV,AA,d)
% Displays 2D projections (cuts) of 3D volume VV
% with activation map AA. 
% While permitting user to navigate around the volume by mouse-clicking directly
% on the projections or moving sliders
%
% d=[dx dy dz] gives sizes of voxels in respective dimensions, defaults
% to [1 1 1]
%
% VV is displayed in gray scale when AA is between 0 and 1 
% for AA between 1 and 2, colours between blue-red are used
%
%
% Jan Kybic, December 1998

global V A h1 h2 h3 h5 h6 h7 h10 c a1 a2 a3 dims maxcol gain ;

if nargin<3, d=[1 1 1] ; end ;

% This is a trick to put all the callback routines in one file
if (isstr(VV)),
  if (strcmp(VV,'slider')),
    disptritwoslider ;
  elseif (strcmp(VV,'show')),
    disptritwoshow ;
  elseif (strcmp(VV,'mouse')),
    disptritwomouse ;
  end ;
else

dims=d ;
% size of the window
x1=1000 ; y1=800 ;
ew=70 ; % element width
eh=20 ; % element height
es=25 ; % element step
gain=1 ;

maxcol=length(jet) ; % colormap size
V=VV-min(min(min(VV))) ;
V=V/max(max(max(V))) ;
A=AA ;

po='Position' ; pr='Parent' ;

% Populate window
h0=gcf ; clf ; zoom off ;
hp=get(h0,po) ;
% x1=hp(3) ; y1=hp(4) ;
hp(3)=x1 ; hp(4)=y1 ;
set(h0,po,hp) ;
set(gcf,'resize','off') ;

xo=x1/2+20 ; yo=y1/2-10 ; x1=xo+ew+10 ; x2=xo+ew*2+20 ;
subplot(221) ; fgr=gcf ; a1=gca ;
subplot(222) ; a2=gca ; 
subplot(223) ; a3=gca ;
h1 = uicontrol(pr,h0,po,[xo yo-es ew eh],'Style','Slider','Tag','Slider1');
h2 = uicontrol(pr,h0,po,[xo yo-2*es ew eh],'Style','Slider','Tag','Slider2');
h3 = uicontrol(pr,h0,po,[xo yo-3*es ew eh],'Style','Slider','Tag','Slider3');
h4 = uicontrol(pr,h0,po,[xo yo-4*es ew eh],'String','Quit','Tag','Pushbutton1');

h5=uicontrol(pr,h0,po,[x1 yo-es ew eh],'Style','Text','Tag','Stext1') ;
h6=uicontrol(pr,h0,po,[x1 yo-2*es ew eh],'Style','Text','Tag','Stext2') ;
h7=uicontrol(pr,h0,po,[x1 yo-3*es ew eh],'Style','Text','Tag','Stext3') ;

h8 = uicontrol(pr,h0,po,[x2 yo-es ew eh],'String','Slice','Tag','Pushbutton2');
h9 = uicontrol(pr,h0,po,[x2 yo-2*es ew eh],'String','Integral','Tag','Pushbutton3');
h10 = uicontrol(pr,h0,po,[x2 yo-3*es ew eh],'Style','Slider','Tag','Slider4');


sv=size(V) ; c=round(sv/2) ;

set(h1,'Min',1) ; set(h1,'Max',sv(1)) ; 
set(h2,'Min',1) ; set(h2,'Max',sv(2)) ;
set(h3,'Min',1) ; set(h3,'Max',sv(3)) ;
set(h10,'Min',-1) ; set(h10,'Max',1) ;
set(h10,'Value',log(gain)/log(10)) ; 

set(h1,'Callback','disptritwo1(''slider'')') ;
set(h2,'Callback','disptritwo1(''slider'')') ;
set(h3,'Callback','disptritwo1(''slider'')') ;
set(h10,'Callback','disptritwo1(''slider'')') ;

set(h4,'Callback','clf ; return ;') ;

set(h8,'Callback','') ;
set(h9,'Callback','') ;

set(fgr,'WindowButtonUpFcn','disptritwo1(''mouse'')') ;

disptritwo1('show');

end ;

%----------------------------------------------------------------------

function updatetexts ;
global c h5 h6 h7 ;
set(h5,'String',num2str(c(1))) ;
set(h6,'String',num2str(c(2))) ;
set(h7,'String',num2str(c(3))) ;


%----------------------------------------------------------------------

function disptritwoslider ;
% Callback for disptritwo
global V h1 h2 h3 h10 c gain ;
c(1)=round(get(h1,'Value')) ;
c(2)=round(get(h2,'Value')) ;
c(3)=round(get(h3,'Value')) ;
gain=10^get(h10,'Value') ;
disptritwoshow ;

%----------------------------------------------------------------------

function supimage(x,aa) ;
global maxcol gain
% superimposes two images as detailed above using image
map=double(aa==0) ;
map1=double(aa<0) ;
map2=1-map1-map ;
[ny,nx]=size(x) ;
% aa=min(ones(ny,nx)*(maxcol-1),aa) ;
% aa=max(zeros(ny,nx),aa) ;
y=zeros(size(x,1),size(x,2),3) ;
cblue=[0 0 1] ; cyellow=[1 1 0] ; cred=[1 0 0] ;
for k=1:3,
  y(:,:,k)=map.*x+map1.*(-max(aa*gain,-1).*cblue(k))+...
      map2.*(min(aa*gain,1).*cred(k)) ;
end ;
image(y) ;

%----------------------------------------------------------------------

function disptritwoshow ;
% helper function for disptritwo
% show V cut at c
global V A h1 h2 h3 c dims;
sv=size(V) ;
colormap(bone) ;
subplot(221) ; supimage(V(:,:,c(3))',A(:,:,c(3))') ; axis image ; axis off ;
set(gca,'DataAspectRatio',[ dims(2) dims(1) 1 ]) ;

hold on ; plot([1 sv(1)],[c(2) c(2)],'r-') ;
plot([c(1) c(1)],[1 sv(2)],'r-') ; hold off ;

subplot(222) ; 
supimage(reshape(V(c(1),:,:),sv(2),sv(3)),reshape(A(c(1),:,:),sv(2),sv(3))); 
axis image ; axis off ;
set(gca,'DataAspectRatio',[ dims(2) dims(3) 1 ]) ;

hold on ; plot([1 sv(3)],[c(2) c(2)],'r-') ;
plot([c(3) c(3)],[1 sv(2)],'r-') ; hold off ;

subplot(223) ; 
supimage(reshape(V(:,c(2),:),sv(1),sv(3))', ...
    reshape(A(:,c(2),:),sv(1),sv(3))') ; 
axis image ; axis off; 
set(gca,'DataAspectRatio',[ dims(3) dims(1) 1 ]) ;
hold on ; plot([1 sv(1)],[c(3) c(3)],'r-') ;
plot([c(1) c(1)],[1 sv(3)],'r-') ; hold off ;

set(h1,'Value',c(1)) ;
set(h2,'Value',c(2)) ;
set(h3,'Value',c(3)) ;
updatetexts ;

%----------------------------------------------------------------------

function disptritwomouse
global a1 a2 a3 c;

a=gca ;
x=get(a,'CurrentPoint') ;
x=round(x) ; 
x=x(1,1:2) ;
if a==a1, c=[x c(3)] ; end ;
if a==a2, c=[c(1) x(2) x(1)] ; end ;
if a==a3, c=[x(1) c(2) x(2)] ; end ;
disptritwoshow ;
