function y=imrexpg(x,n,order)
% Given an image, described by cubic/quadratic/linear B-spline coefficients
% with shift 0.5, enlarge each row to n columns and return in y.
%
% Now uses 8 boundary points, i.e. relative error should be <1e-6
%
% Usage: y=imrexp(x)

[ny,nx]=size(x) ;
y=zeros(ny,n) ;

if order==3,
  shift=1 ; ofs=10 ; 
  step=(nx-2*shift-1)/(n-2*shift-1) ;
  t=1+shift-shift*step-ofs*step:step:nx-shift+shift*step+ofs*step ;
  for i=1:ny,
    v=mcbinterp(x(i,:),t) ;
    v=cbanal(v) ;
    y(i,:)=v(ofs+1:length(v)-ofs) ;
  end ;
elseif order==2,
  shift=0.5 ; ofs=10 ; 
  step=(nx-2*shift-1)/(n-2*shift-1) ;
  t=1+shift-shift*step-ofs*step:step:nx-shift+shift*step+ofs*step ;
  for i=1:ny,
    v=mqbinterp(x(i,:),t) ;
    v=qbanal(v) ;
    y(i,:)=v(ofs+1:length(v)-ofs) ;
  end ;
else
  shift=0 ; ofs=0 ; 
  step=(nx-2*shift-1)/(n-2*shift-1) ;
  t=1+shift-shift*step-ofs*step:step:nx-shift+shift*step+ofs*step ;
  for i=1:ny,
    v=mlbinterp(x(i,:),t) ;
    y(i,:)=v(ofs+1:length(v)-ofs) ;
  end ;
end ;


  