% Haar smoothing using Kalman filters

% Generate data
N=101 ; % data length
t=1:N ;
M=10 ; % subsampling factor
yid=sin((1:N)/N*5) ; % ideal
sigma=0.5 ; % noise amplitude
y=sigma*randn(1,N)+yid ;
% plot(t,yid,t,y) ;
% legend('ideal','measured') ;

A=ones(N,1) ;  % system matrix
B=zeros(N,1) ; % input matrix
for i=1:M:N, % transition occur between M+1, M+2; 2M+1, 2M+2 
  A(i)=0 ; B(i)=1 ;
end ;

Pb=zeros(N,1) ; % a priori covariance
Pa=zeros(N,1) ; % a posteriori covariance
xfa=zeros(N,1) ; % forward run estimation a posteriori
xfb=zeros(N,1) ; % forward run estimation a priori
xb=zeros(N,1) ; % backward run estimation

% Initialize forward run
xfb(1)=0 ; Pb(1)=1e5 ; % uniformative a priori pdf
su=30^2 ; % system noise cov.
sv=0.2^2 ; % measurement noise cov.

% Forward run
for i=1:N,
  % data step
  L=Pb(i)/(Pb(i)+sv) ;
  xfa(i)=xfb(i)+L*(y(i)-xfb(i)) ;
  Pa(i)=Pb(i)-L*Pb(i) ;
  % time step
  if i<N,
    xfb(i+1)=A(i)*xfa(i) ;
    Pb(i+1)=A(i)*Pa(i)*A(i)'+su*B(i) ;
  end ;
end ;

yh=zeros(N,1) ; % Haar smoothing Kalman results
yhr=zeros(N,1) ; % Haar smoothing optimal results
for i=1:N,
  j=min(ceil((i-1)/M)*M+1,N) ;
  yh(i)=xfa(j) ;
  if j>1, 
    yhr(i)=mean(y(j-M+1:j)) ;
  else
    yhr(i)=y(i) ;
  end ;
end ;




subplot(211) ;
plot(t,y,t,xfa,t,yh,t,yhr) ;
legend('measured','filtered','Haar','opt Haar') ;
subplot(212) ;
% plot(2:N,Pb(2:N)/(Pb(2:N)+sv)) ;
plot(t,yh-yhr) ;

