% Linear spline smoothing using Kalman filters

% Generate data
N=101 ; % data length
t=1:N ;
M=20 ; % subsampling factor
% yid=sin((1:N)/N*5) ; % ideal
yid=[ 1:50 51:-1:1 ] ;
sigma=0 ; % noise amplitude
y=sigma*randn(1,N)+yid ;
% plot(t,yid,t,y) ;
% legend('ideal','measured') ;

A=zeros(2,2,N) ;  % system matrix
B=zeros(2,N) ; % input matrix
for i=1:N,
  if rem(i,M)==0, % knot
    A(:,:,i)=[ 0 0 ; 0 1 ] ; B(:,i)=[ 1 0 ]' ;
  else
    A(:,:,i)=[ 1 0 ; 1 1 ] ; B(:,i)=[ 0 0 ]' ;
  end ;
end ;
C=[0 1] ;

Pb=zeros(2,2,N) ; % a priori covariance
Pa=zeros(2,2,N) ; % a posteriori covariance
xfa=zeros(2,N) ; % forward run estimation a posteriori
xfb=zeros(2,N) ; % forward run estimation a priori
xb=zeros(2,N) ; % backward run estimation

% Initialize forward run
xfb(:,1)=[0 0]' ; Pb(:,:,1)=1e5*eye(2) ; % uniformative a priori pdf
su=10^2 ; % system noise cov.
sv=10^2 ; % measurement noise cov.

% Forward run
for i=1:N,
  % data step
  L=Pb(:,2,i)/(Pb(2,2,i)+sv) ;
  xfa(:,i)=xfb(:,i)+L*(y(i)-xfb(2,i)) ;
  Pa(:,:,i)=Pb(:,:,i)-L*C*Pb(:,:,i) ;
  % time step
  if i<N,
    xfb(:,i+1)=A(:,:,i)*xfa(:,i) ;
    Pb(:,:,i+1)=A(:,:,i)*Pa(:,:,i)*A(:,:,i)'+su*B(:,i)*B(:,i)' ;
  end ;
end ;

xb(:,N)=xfa(:,N) ;

% Backward run
for i=N-1:-1:1,
  F=Pa(:,:,i)*A(:,:,i+1)'/Pb(:,:,i+1) ;
  xb(:,i)=xfa(:,i)-F*(xb(:,i+1)-xfb(:,i+1)) ;
  % plot(i:N,xb(2,i:N),i:N,xfa(2,i:N),i:N,y(i:N)) ;
  % legend('backward','filtered','measured') ;
  % pause(0.01) ;
end ;  

% subplot(211) ;
plot(t,y,t,xfa(2,:),t,xb(2,:)) ;
legend('measured','filtered','backward') ;
% subplot(212) ;
% plot(2:N,Pb(2:N)/(Pb(2:N)+sv)) ;
% plot(t,yh-yhr) ;

