% Linear spline smoothing using Kalman filters
% uses the kalmtvf subroutine

% Generate data
N=101 ; % data length
t=1:N ;
M=20 ; % subsampling factor
% yid=sin((1:N)/N*5) ; % ideal
yid=[ 2:20 21:-1:2 1:20 21:-1:2  1:22 ] ;
sigma=0 ; % noise amplitude
y=sigma*randn(1,N)+yid ;
% plot(t,yid,t,y) ;
% legend('ideal','measured') ;

A=zeros(2,2) ;  % system matrix
Q=zeros(2,2,N) ; % system noise covariance matrix
R=zeros(1,1,N) ; % measurement noise covariance matrix
A=[1 0 ; 1 1] ;
for i=1:N,
  if rem(i,M)==0, % knot
    Q(:,:,i)=[ 1 0 ; 0 0 ]*1e5 ;
  else
    Q(:,:,i)=zeros(2,2) ;
  end ;
  R(:,:,i)=0.001 ; 
end ;
C=[0 1] ; % output matrix

% Initialize forward run
xs(:,1)=[0 0]' ; Ps=1e15*eye(2) ; % uniformative a priori pdf

[xa,xb,Pa,Pb]=kalmtvf(A,C,Q,R,y,Ps,xs) ;

% Now the backward run
Ab=inv(A) ; 
Qr=zeros(2,2,N) ;
for i=1:N-1,
  Qr(:,:,N-i+1)=Ab*Q(:,:,i+1)*Ab' ;
end ;

[xab,xbb,Pab,Pbb]=kalmtvf(Ab,C,Qr,R,y(N:-1:1),Ps,xs) ;
xrb=xbb(2,N:-1:1) ;
xra=xab(2,N:-1:1) ;

% Compose estimates of the two runs
Pbackwb=reshape(Pbb(2,2,N:-1:1),1,N) ;
Pbackwa=reshape(Pab(2,2,N:-1:1),1,N) ;
Pforwa=reshape(Pa(2,2,:),1,N) ;
Pforwb=reshape(Pb(2,2,:),1,N) ;

yc=zeros(1,N) ;
Pc=zeros(1,N) ;

for i=1:N,
  Pc1=(Pforwa(i)+Pbackwb(i)) ;
  Pc2=(Pforwb(i)+Pbackwa(i)) ;
  if Pc1>Pc2,
    yc(i)=(Pbackwa(i)*xb(2,i)+Pforwb(i)*xra(i))/Pc2 ;
    Pc(i)=Pc2 ;
  else
    yc(i)=(Pbackwb(i)*xa(2,i)+Pforwa(i)*xrb(i))/Pc1 ;
    Pc(i)=Pc1 ;
  end ;
end ;

figure(1) ; 
subplot(211) ;
plot(t,yid,t,y,t,xa(2,:),t,xr,t,xc) ;
legend('ideal','measured','filtered','backward','composed') ;
subplot(212) ;
plot(t,yid-xa(2,:),t,yid-xr,t,yid-xc) ;
legend('forward','backward','composed') ;

figure(2) ; 
subplot(221) ;
plot(t,yid,t,y,t,xc) ;
legend('ideal','measured','composed') ;
subplot(222) ;
plot(t,yid-xc) ;
legend('composed') ;
subplot(223) ;
plot(2:N-2,Pc(2:N-2)) ;
legend('composed uncertainty') ;


