function [xa,xb,Pa,Pb]=kalmtvf(A,C,Q,R,y,Ps,xs) ;
% Kalman forward filter for time-variant input-less linear system
%   x(:,t+1)=A*x(:,t)+u(t+1)
%   y(:,t)=C*x(:,t)+v(t)
%
%  where u,v are Gaussian i.i.d. white noises with covariances
%  Q(:,:,t) and R(:,:,t) 
%
% Returns the LMS a predicted (xb) and filtered (xa) estimate
% of the state x at all times, given a priori values Ps,xs

[n,m]=size(A) ; % get system order 
if n~=m,
  error('A must be n*n') ;
end ;

[m,T]=size(y) ; 

Pb=zeros(n,n,T) ; % a priori covariance
Pa=zeros(n,n,T) ; % a posteriori covariance
xa=zeros(n,T) ; % forward run estimation a posteriori
xb=zeros(n,T) ; % forward run estimation a priori

Pb(:,:,1)=Ps ; % starting guess
xb(:,1)=xs ;

% Forward run
for i=1:T,
  % data step: first calculate Kalman gain
  L=Pb(:,:,i)*C'/(C*Pb(:,:,i)*C'+R(:,:,i)) ;
  xa(:,i)=xb(:,i)+L*(y(i)-C*xb(:,i)) ;
  Pa(:,:,i)=Pb(:,:,i)-L*C*Pb(:,:,i) ;
  % time step, unless we are at the end
  if i<T,
    xb(:,i+1)=A*xa(:,i) ;
    Pb(:,:,i+1)=A*Pa(:,:,i)*A'+Q(:,:,i+1) ;
  end ;
end ;

