function c=lndunicoef(x,z) ;
% Calculate coefficients for unidirectional landmark warping function
% The R^2->R function w is constrained to be z() at points x()
% x contains (x,y) coordinate pair in each row
% Elsewhere, it is a linear combination of r^2 log r 
% basis functions
%
% Returns the coefficients c of w in this basis.
%
% Usage: function c=lndunicoef(x,z) ;

w=0 ;
n=length(z) ;
A=zeros(n) ;

for i=1:n,
  for j=1:n,
    r2=(x(i,1)-x(j,1))^2+(x(i,2)-x(j,2))^2 ;
    if r2==0,
      A(i,j)=0 ;
    else
      A(i,j)=r2*log(r2) ; % actually twice as much as r^2 ln(r) 
    end ;
  end ;
end ;

% Solve using SVD techniques
[u,s,v]=svd(A) ;
s=diag(s) ; q=(s(1)*1e-6+1e-50) ;
for i=1:n,
  if s(i)<q, s(i)=0 ; 
  else s(i)=1/s(i) ; end ;
end ;

c=v*diag(s)*u'*z(:) ;
% c=A\z(:) 
