/* mexlnduniimgwarp.c MEX file source

   Given a radial basis function knots and coefficients,
   warp an image given its B-spline interpolation coefficients f

   Usage: y=mexlnduniimgwarp(f,x,c)

   Jan Kybic, March 1999

   Uses fi(r)= 2 r^2 ln(r)   basis functions

*/


#define myCreateMatrix(a,b) mxCreateDoubleMatrix(a,b,mxREAL)
#define myAlloc(a) mxMalloc(a)

#include <math.h>
#include "mex.h"

#define	F_IN	prhs[0]
#define X_IN	prhs[1]
#define C_IN	prhs[2]
#define Y_OUT   plhs[0]

#define NAME "MEXLNDUNIIMWARP"

#define max(a,b) ((a>b) ? (a) : (b))

void  mexlnduniimgwarp(double *yp,double *fp,double *xp,double *cp,
		       int ny,int nx,int n) ;

void mexFunction(int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{ int nx,ny,n ; int i ;
  double *fp,*xp,*cp,*yp ;
 
  if (nrhs != 3) {
    mexErrMsgTxt(NAME " requires three input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }
  
  for (i=0;i<nrhs;i++)
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(
             NAME " needs all input arguments to be real.");
     } 
 
  ny=mxGetM(F_IN) ; nx=mxGetN(F_IN) ;
  n=mxGetM(X_IN) ;
  fp=mxGetPr(F_IN) ;  xp=mxGetPr(X_IN) ;   cp=mxGetPr(C_IN) ; 

  Y_OUT = myCreateMatrix(ny,nx); yp=mxGetPr(Y_OUT) ;
 /*  %%%%% here the calculation begins %%%%%% */
  mexlnduniimgwarp(yp,fp,xp,cp,ny,nx,n) ;
} 

#define SAFEMALLOC(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(double)))==NULL) \
    mexErrMsgTxt(NAME ": could not malloc") ;

#define SAFEMALLOCINT(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(int)))==NULL) \
    mexErrMsgTxt(NAME ": could not malloc") ;


double cbinterp(double *c,int nc,double x) ;

void  mexlnduniimgwarp(double *yp,double *fp,double *xp,double *cp,
		       int ny,int nx,int n) 
{ 
  int i,j,k ; double r,rx,t1,t2,*tf ;

  SAFEMALLOC(tf,ny,nx) ;
  /* transpose fp */
  for(i=0;i<nx;i++) /* iterate over columns */
    for(j=0;j<ny;j++) /* iterate over rows */
      tf[j*nx+i]=fp[j+ny*i] ;

   for(j=0;j<ny;j++) { /* iterate over rows */
     for(i=0;i<nx;i++) { /* iterate over columns */
       rx=(double)(i+1) ; /* new x coordinate */
       for(k=0;k<n;k++) { /* loop over all basis functions */
	 /* The 1+ below is because c's were computed in Matlab coordinates */
	 t1=(1+i-xp[k]) ; t2=(1+j-xp[k+n]) ; r=t1*t1+t2*t2 ;
	 if (r>0.0) rx+=cp[k]*r*log(r) ;
       }
       yp[j+ny*i]=cbinterp(tf+j*nx,nx,rx) ;
     }
   }
  mxFree(tf) ; 
}

#include "bsplinetools.c"

