function [c,E]=multirespyr(frp,ftp,s) ;
% Given a test image pyramid ftp and reference image pyramid frp, 
% find warp coefficients c
% so that the total square difference E between the reference image and
% warped test image is minimal.
% Number of mesh coefficients can be now arbitrary
% 
% For description of s, see spwstruct.m
%
% fr and ft are two dimensional, with the same number of rows. Warping takes
% place across columns. 
% 
% No multiresolution in warp space 

global wwarr ww mask wtimes
fr=frp{1} ; ft=ftp{1} ;
% the pyramide level count
nl=size(frp,1) ;

imaging=s(12)>0 ;
dispwfflg=s(13)>0 ; % show warping function
wfcnt=1 ;
nc=s(2) ; nd=s(3) ;
degree=s(1) ;

nca=nc ; nda=nd ;

cmsf=length(colormap) ;

[ony onr]=size(fr) ; 
[fullny fullnr]=size(fr) ;

for i=nl:-1:1,
  frc=frp{i} ; ftc=ftp{i} ;
  if i~=nl, 
    oldnr=nr ; 
  end ;
  ny=size(frc,1) ; nr=size(frc,2) ; nt=size(ftc,2) ;
  disp([ 'Level ' num2str(i) ' rows=' num2str(ny) ' colref=' num2str(nr) ...
	' coltest=' num2str(nt) ' nca=' num2str(nca) ' nda=' num2str(nda)]) ;

  if imaging,  
    subplot(221) ; image(cmsf*frc) ; title('reference') ; colorbar ; 
    subplot(222) ; image(cmsf*ftc) ; title('test') ; colorbar ;
    colormap(jet) ;
    pause(0.01) ;  
  end ;  

  if i==nl, % The first time -> initialize c
    c=zeros(nd,nc) ; nx=nr ;
    step=(nx-1)/(nc-3) ;
    for y=1:nd ;
      c(y,:)=linspace(1-step,nx+step,nc) ;
      c(y,:)=cbanal(c(y,:)) ;
    end ;
    for x=1:nc,
      c(:,x)=cbanal(c(:,x)')' ;
    end ;
  else
    % image dimension was increased, change c appropriately
    gamma=(nr-1)/(oldnr-1) ;
    delta=1-gamma ;
    c=delta+gamma*c ;
  end ;


  
  % Optimize 
  gamma=(128-1)/(nr-1) ;
  delta=1-gamma ;
  cc=cputime ;
  [c,E]=spwopt(frc,ftc,c,s) ;
  wtimes=[wtimes cputime-cc ]
  xs=splnwarp21o(reshape(delta+gamma*c,1,nca*nda),nca,128,128,degree) ;
  mdif=(xs-ww).*double(mask) ;
  wi=sum(sum(abs(mdif)))/sum(sum(mask)) ;
  wwarr=[wwarr wi]
  
  
  if dispwfflg, figure ; dispwcntr(c) ; axis off ; 
        if s(13)>1, eval(['print -depsc demowf' num2str(wfcnt) '.eps']) ; 
      wfcnt=wfcnt+1 ; end ;
  end ;
  
  imeb=imsanal(ftc,s(19)) ;
  imh=meximwarpd(imeb,reshape(c,1,size(c,1)*size(c,2)),nr,nca,s(1),s(19)) ;

  if imaging,  
    subplot(223) ; image(cmsf*imh) ; title('warped') ; colorbar ; grid on ;
    subplot(224) ; image(cmsf*((imh-frc)/2+0.5)) ; title('difference') ; 
    colorbar ; grid on ;
    % disp('Press a key') ; 
    pause(0.01) ;
  end ;

end ; % main loop - all pyramid levels 

% Now, if we have not yet reached the desired resolution, go for it at once

% disp([ 'Main loop finished at nca=' num2str(nca) ' nda=' num2str(nda) ]) ; 
% disp([ 'Expanding to nc=' num2str(nc) ' nd=' num2str(nd) ]) ; 

if(nca<nc | nda<nd),
  
  if nca<nc, 
    nca=nc ; 
    disp([ 'Expanding parameter space horizontally: nca=' num2str(nca)]) ;
    c=imrexpg(c,nc,degree) ;
  end ;  

  if nda<nd, % 4*ndn<=ny,
    nda=nd ;
    disp([ 'Expanding parameter space vertically: nda=' num2str(nda)]) ;
    c=imrexpg(c',nd,degree)' ;
  end ;
  
  % Optimize 
  if dispwfflg, figure ; dispwcntr(c) ; axis off ; 
        if s(13)>1, eval(['print -depsc demowf' num2str(wfcnt) '.eps']) ; 
      wfcnt=wfcnt+1 ; end ;
    end ;
  cc=cputime ;  
  [c,E]=spwopt(frc,ftc,c,s) ;
  wtimes=[wtimes cputime-cc ]
  imh=meximwarp(imeb,reshape(c,1,size(c,1)*size(c,2)),nr,nca,s(1)) ;
  xs=splnwarp21o(reshape(delta+gamma*c,1,nca*nda),nca,128,128,degree) ;
  mdif=(xs-ww).*double(mask) ;
  wi=sum(sum(abs(mdif)))/sum(sum(mask))  ;
  wwarr=[wwarr wi]

  if imaging,
    subplot(223) ; image(cmsf*imh) ; title('warped') ; colorbar ; grid on ;
    subplot(224) ; image(cmsf*((imh-frc)/2+0.5)) ; title('difference') ; 
    colorbar ; grid on ;
    % disp('Press a key') ; 
    pause(0.01) ;
  end ;
end ;

if dispwfflg, figure ; dispwcntr(c) ; axis off ; 
      if s(13)>1, eval(['print -depsc demowf' num2str(wfcnt) '.eps']) ; 
      wfcnt=wfcnt+1 ; end ;
end ;



