function y=addconsmon(x,z)
% Returns y=x+z, or an approximation of y so that 
% y is guaranteed to be non-decreasing
% x must be non-decreasing

n=length(x) ;
if length(z)~=n,
  error('x and z must have the same length.') ;
end ;

relprec=sum(abs(x))/1e-6 ;

while (1),

  lambda=1 ; activei=0 ;
  for i=2:n,
    dx=x(i)-x(i-1) ;
    if dx<0,
      warning('x is decreasing - correcting.') ;
      x(i)=x(i-1) ; dx=0 ;
    end ;
    dz=z(i)-z(i-1) ; 
    if dz<0,
      l=-dx/dz ;
      if l<lambda,
	lambda=l ; activei=i ;
      end ;
    end ; % if dz<0
  end ;   % for

  % x,z
  % activei, lambda
  % pause ;
  % activei now contains the number of the active limitation
  
  % go as far as possible without breaking the constraints
  x=x+lambda*z ; 
   % if no more limitations are active, we are finished
  if activei==0, break ; end ;

  % now z has to be modified not to point perpendicularly to the constraint
  z=z-lambda*z ;
  k=(z(activei-1)+z(activei))/2 ;
  z(activei)=k ; z(activei-1)=k ;

end ;

y=x ;