function cardinal=L2spyramid_up(dual,n)
% L2SPYRAMID_UP     Image EXPAND procedure
%   L2SPYRAMID_UP(dual,n) upsamples the image dual
%   by a factor of 2 using spline filtering (n n). 
%   The input image is in the dual space and the output image in the cardinal space.  
%
%   Reference: IEEE Vol15, No.4, p 371, fig 3.
%   Author: Stefan Horbelt, 23.4.1998

xsize=size(dual,1);
ysize=size(dual,2);

img=zeros(xsize,ysize);
up=zeros(xsize*2,ysize*2);
cardinal=zeros(xsize*2,ysize*2);

% dual->B-spline: (b^(2n+1))^-1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for x=1:xsize
  img(x,:)=fspline( dual(x,:),2*n+1);
end;
for y=1:ysize
  img(:,y)=fspline( img(:,y),2*n+1);
end;


% EXPAND (B-spline basis)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


up=dyadup(img,'m',1);
up=up(2:xsize*2+1,2:ysize*2+1);


u=bio_coeff(n);

for x=1:xsize*2
  up(x,:)=filfirs(up(x,:),u);
end;

for y=1:ysize*2
  up(:,y)=filfirs(up(:,y),u);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%ok

fb=fbspline(n);

for y=1:xsize*2
  up(:,y)=filfirs(up(:,y),fb);
end;

for x=1:ysize*2
  cardinal(x,:)=filfirs(up(x,:),fb);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


