function dual=L2spyramid_down(cardinal,n, varargin)
% L2SPYRAMID_DOWN     Image REDUCE procedure
%   L2SPYRAMID_DOWN(cardinal,n[,'dual'|'basic'|'cardinal']) downsamples the image cardinal
%   by a factor of 2 using spline filtering (n n). The input image is in 
%   the cardinal space and the output image in the dual space.
%   Option: input image is assumed to be 'cardinal' can be in 'dual' or 'basic' space.
%
%   Reference: IEEE Vol15, No.4, p 371, fig 3.
%   Author: Stefan Horbelt, 23.4.1998

xsize=size(cardinal,1);
ysize=size(cardinal,2);

%if(mod(xsize,2)~=0 | mod(ysize,2)~=0 | xsize*ysize<=4)
%  error('ERROR: image size must be multiple of 2'); 
%end;

img=cardinal;


if(nargin>=3)
     space=varargin{1}(1:4);
else
  space='card';
end;

space=space(1:4);

% Cardinal -> Basic: (b^n)^-1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%ok
if(space=='card')
  for x=1:xsize
   img(x,:)=fspline(img(x,:),n);
  end;

  for y=1:ysize
   img(:,y)=fspline(img(:,y),n);
  end;
  space='basi';
end;

% Basic -> Dual : (b^2n+1)^-1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(space=='basi')
  fb2=fbspline(n*2+1);

  for x=1:xsize
    img(x,:)=filfirs(img(x,:),fb2);
  end;

  for y=1:ysize
    img(:,y)=filfirs(img(:,y),fb2);
  end;
  space='dual';
end;

% REDUCE (dual basis) prefilter: 1/2*u2^n
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(space=='dual')
  u=bio_coeff(n);

  for x=1:xsize
    img(x,:)=filfirs(img(x,:),u/2);
  end;

  for y=1:ysize
    img(:,y)=filfirs(img(:,y),u/2);
  end;

  dual=dyaddown(img,'m',1);
end;

