function out=spspace(img, n, spacein, spaceout)
% SPSPACE  Changes the space of a polynomial spline representation
%   Using filter of order n; 
%   Input image can be in 'cardinal', 'dual' or 'bspline' space.
%   Output image can be in 'cardinal', 'dual' or 'bspline' space.
%
%   Reference: IEEE Vol15, No.4, p 371, fig 3.
%   Author: Stefan Horbelt, 23.4.1998

xsize=size(img,1);
ysize=size(img,2);

spacein= spacein(1:4);
spaceout=spaceout(1:4);

% Cardinal -> Bspline: (b^n)^-1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%ok
if(spacein=='card' & spaceout~='card')

  for x=1:xsize
   img(x,:)=fspline(img(x,:),n);
  end;

  for y=1:ysize
   img(:,y)=fspline(img(:,y),n);
  end;
  spacein='bspl';
end;

% Bspline -> Dual : (b^2n+1)^-1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(spacein=='bspl' & spaceout=='dual')

  fb2=fbspline(n*2+1);

  for x=1:xsize
    img(x,:)=filfirs(img(x,:),fb2);
  end;

  for y=1:ysize
    img(:,y)=filfirs(img(:,y),fb2);
  end;
  spacein='dual';
end;

% Dual->B-spline: (b^(2n+1))^-1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (spacein=='dual' & spaceout~='dual')
  for x=1:xsize
    img(x,:)=fspline( img(x,:),2*n+1);
  end;
  for y=1:ysize
    img(:,y)=fspline( img(:,y),2*n+1);
  end;
  spacein='bspl';
end;

% B-spline->cardinal
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (spacein=='bspl' & spaceout=='card')

  fb=fbspline(n);

% swapped ysize,xsize here - JK  
  for y=1:ysize
    img(:,y)=filfirs(img(:,y),fb);
  end;

  for x=1:xsize
    img(x,:)=filfirs(img(x,:),fb);
  end;
end;

out=img;


