function [c,E]=imwarpopti(fr,ft,nc,nd,c,order) ;
% Given a test image ft and reference image fr, find warp coefficients c
% so that the total square difference E between the reference image and
% warped test image is minimal
% nc is the number of nodes interpolating the warping function horizontally
% and nd vertically
%
% Uses Levenberg-Marquardt algorithm inspired by Numerical Recipies
% Does not check for constraints but uses penalty function instead
% Now uses quadratic B-spline interpolation for the warping function g
%
% fr and ft are two dimensional, with the same number of rows. Warping takes
% place across columns
%
% c is the initial estimate of the warping coefficients
% order=1(linear),2(quadratic),3(cubic)
%
% Usage: [c,E]=imwarpopti(fr,ft,nc,nd,c,order) ;

displaying=0 ; % should progress messages be printed

% Tlambda=0 ; % coefficient for the regularization Tikhonov stabilizer
Tlambda=5e-6 ;

[ny nr]=size(fr) ;
[nyt nt]=size(ft) ;

if ny~=nyt, error('fr and ft must have the same number of rows') ; end ;

if order==2,
  if nc<4 & nd<4, error('nc,nd must be at least 4.') ; end ;
end ;


ftb=imcbanal(ft) ;

% [Eold,gr,H]=meximwarpcalcbddh(c,fr,ft,ftb,nc) ;
[Eold,gr,H]=meximwarpcalc(c,fr,ft,ftb,nc,order,Tlambda) ;

E=Eold ;
Eold=2*Eold+1 ;
prec=1e-4   ;
delta=1e-10 ;
lambda=0.1 ;
i=1 ; j=0 ;

imh=meximwarp(ftb,c,nr,nc,order) ;


Ein=sum(sum((imh-fr).^2)) ;

while (Eold-E>Eold*prec+delta | j<1), 
  % previously while ((Eold-E)/Eold>prec | j<3),
  if displaying,   
    disp([ 'Iteration ' num2str(i) '  criterion=' num2str(E)]) ; 
  end ;

  E1=E ;
  M=H+eye(nc*nd)*lambda ; rc=rcond(M) ;
  if 1, % classical versus SVD
    if rc>1e-10,
      dc=-gr/M' ;
      cpdc=c+dc ;
      [E,gr1,H1]=meximwarpcalc(cpdc,fr,ft,ftb,nc,order,Tlambda) ;
      
      if displaying,
	disp(['Returned E=' num2str(E)]) ;
      end ;	
      i=i+1 ;
    else
      if displaying,
	disp('Linear equation set close to singular.') ;
      end ;
    end ;
  end ;
    
  if E>E1 | rc<=1e-10,
    lambda=10*lambda ; E=E1 ;
    if displaying,
      disp(['Increasing lambda and trying again lambda=' num2str(lambda)]) ;
    end ;  
    j=0 ;
  else
    lambda=lambda/10 ;
    c=cpdc ;
    Eold=E1 ; H=H1 ; gr=gr1 ;
    j=j+1 ;
  end ;
end ;

if displaying,
  disp([ 'Iterations ' num2str(i) '  criterion=' num2str(E)]) ;
end ;

disp([ 'Iterations ' num2str(i) '  rel. criterion=' ...
      num2str(E/size(fr,1)/size(fr,2))]) ;
imh=meximwarp(ftb,c,nr,nc,order) ;


Eout=sum(sum((imh-fr).^2)) ;
disp([ 'Criterion ' num2str(Ein) ' -> ' num2str(Eout) ]) ;
