/*
  meximwarp    MEX file source

  Given a 2D function sampled at discrete points given by its cubic
  B-spline interpolation coefficients f and a 2D warping function
  represented by linear/quadratic/cubic B-spline interpolation
  coefficients c at discrete equidistant points (plus one at each
  side), return a new image y of n columns. The number of rows stays
  unchanged
 
  Currently uses cubic B-spline interpolation for the images.
  of columns given by nc

  Calling: y=meximwarp(f,c,n,nc,degree)


  Jan Kybic, May 1997

*/

/* #ifdef __GNUC__ 
 #ifndef linux 
 #define linux 
 #endif 
 #endif */

#ifdef linux
#define myCreateMatrix(a,b) mxCreateFull(a,b,REAL)
#define myAlloc(a) mxCalloc(a,1)
#else /* hp version */
#define myCreateMatrix(a,b) mxCreateDoubleMatrix(a,b,mxREAL)
#define myAlloc(a) mxMalloc(a)
#endif

#include <string.h>
#include <math.h>
#include "mex.h"

/* Input Arguments */

#define	F_IN	prhs[0]
#define	C_IN	prhs[1]
#define	N_IN	prhs[2]
#define NC_IN   prhs[3]
#define DEGREE_IN  prhs[4]


/* Output Arguments */

#define	Y_OUT	plhs[0]

#define NAME "MEXIMWARP"

#define max(a,b) ((a>b) ? (a) : (b))

void imwarpbddh(double *yp,double *fp,double *cp,int n,int nf,int nc,int nd,
	       int ny) ;
void imwarpldd(double *yp,double *fp,double *cp,int n,int nf,int nc,int nd,
	       int ny) ;
void imwarpcdd(double *yp,double *fp,double *cp,int n,int nf,int nc,int nd,
	       int ny) ;

#ifdef linux
void mexFunction(
                 int nlhs, Matrix *plhs[],
                 int nrhs, Matrix *prhs[]
		 )
#else
void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
#endif
{ double *yp,*fp,*cp ;
  int i,n,nf,nc,nd,nct,ny,degree ;

  /* Check for proper number of arguments */
  
  if (nrhs != 5) {
    mexErrMsgTxt(NAME " requires five input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }
  
  for (i=0;i<nrhs;i++)
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(
             NAME " needs all input arguments to be real.");
     } 
  
  ny=mxGetM(F_IN) ; nf=mxGetN(F_IN) ;
  nc=(int)(*(mxGetPr(NC_IN))) ;
  nct=max(mxGetM(C_IN),mxGetN(C_IN)) ;
  nd=nct/nc ;  
  degree=(int)(*(mxGetPr(DEGREE_IN))) ;

  yp = mxGetPr(N_IN); fp=mxGetPr(F_IN) ; cp = mxGetPr(C_IN); 
  n=(int)(*yp) ;

  Y_OUT = myCreateMatrix(ny,n);
  yp=mxGetPr(Y_OUT) ;

  /*  %%%%% here the calculation begins %%%%%% */
  switch (degree) {
  case 1: imwarpldd(yp,fp,cp,n,nf,nc,nd,ny) ;; break ;  
  case 2: imwarpbddh(yp,fp,cp,n,nf,nc,nd,ny) ; break ;
  case 3: imwarpcdd(yp,fp,cp,n,nf,nc,nd,ny) ; break ;
  default: mexErrMsgTxt(NAME ": unsupported degree") ;  
  }
  return;
}

#define SAFEMALLOC(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(double)))==NULL) \
    mexErrMsgTxt(NAME ": could not malloc") ;

#define SAFEMALLOCINT(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(int)))==NULL) \
    mexErrMsgTxt(NAME ": could not malloc") ;

#define MAKING_MEX

#include "bsplinetools.c"

