function [c,E]=spwfind(refim,testim,s) ;
% Given reference image refim and a test image testim, find matrix 
% c of B-spline coefficients describing the warping function
% minimizing the difference between r and t after warping
% For description of s, see spwstruct.m
% Also returns the quadratic aposteriori difference E.
%
% Images are expected to be the same size. 
%
% Jan Kybic, November 1998
%
% Usage: c=spwfind(refim,testim,s) ;

if prod(size(refim)==size(testim))==0,
  error('Images must have the same size.') ;
end ;

cmsf=length(colormap) ;

% Calculate, how many pyramid levels are needed 
% so that the coarsest level has size of at most s(10)^2 pixels
nl=(min(log(length(refim)),log(length(testim)))-log(s(10)))/log(2) ;
nl=round(max(nl,0)) ;

if s(4)==0,
  frefim=refim ; ftestim=testim ;
elseif s(4)==1,
  filtsize=s(5) ;
  frefim=imhpfilter(1/filtsize,refim) ;
  ftestim=imhpfilter(1/filtsize,testim) ;
  ftestim=imhisteq(frefim,ftestim) ;
elseif s(4)==2,
  frefim=locnorm(refim,s(5)) ;
  ftestim=locnorm(testim,s(5)) ;
else
  error('Unknown preprocessing parameter') ;
end ;

if s(12)>1,
 subplot(221) ; image(cmsf*refim) ; title('reference') ;
 subplot(222) ; image(cmsf*testim) ; title('test') ;
 subplot(223) ; image(cmsf*frefim) ; title('filt. ref.') ;
 subplot(224) ; image(cmsf*ftestim) ; title('filt. test') ;
end ;

ctime=cputime ;
refp=pyramid2(frefim,nl) ;
testp=pyramid2(ftestim,nl) ;
if s(11)>0,
  disp([ 'Pyramids took ' num2str(cputime-ctime) ' s' ]) ;
end ;

[c,E]=spwpyr(refp,testp,s) ;

