function ec=expandc(c,nc,x,nx,nnc,nnd) ;
% Having coefficients c (nc columns, nd rows) for picture of x columns 
% expands them to nnc columns and nnd rows for a picture of nx columns
% by quadratic B-spline interpolation
% Both c and ec are stored in vectors.
%
% Usage: ec=expandc(c,nc,x,nx,nnc,nnd) ;

% change c to matrix form
nd=length(c)/nc ;
c=reshape(c,nd,nc) ;
ec=zeros(nnd,nnc) ;
tc=zeros(nd,nnc) ;

% Expand across columns
alpha=(nc-3)/(nnc-3) ;
beta=2-2*alpha ;
for i=1:nd, % for all rows
  for j=1:nnc,
    t=beta+j*alpha ;
    tc(i,j)=qbinterp(c(i,:),t) ;
  end ;
end ;

% Expand across rows

alpha=(nd-3)/(nnd-3) ;
beta=2-2*alpha ;
gamma=(nx-1)/(x-1) ;
delta=1-gamma ;
for j=1:nnc,
  for i=1:nnd,
    t=beta+i*alpha ;
    z=qbinterp(tc(:,j),t) ;
    ec(i,j)=delta+z*gamma ;
  end ;
end ;

% Find interpolating coefficients
ec=imqbanal(ec) ; % by rows
ec=imqbanal(ec')' ; % by columns

% Check for constraints
e=max(max(abs(ec)))*1e-6 ;
for i=1:nnd,
  if ec(i,1)>ec(i,3), ec(i,1)=ec(i,3)-e ; end ;
  for j=3:nnc-1,
    if ec(i,j)<ec(i,j-1),  ec(i,j)=ec(i,j-1)+e ; end ;
  end ;
  if ec(i,nnc)<ec(i,nnc-2), ec(i,nnc)=ec(i,nnc-2)+e ; end ;
end ;

ec=reshape(ec,1,nnc*nnd) ;
