function [E,gr,H]=imwarpcalcb(c,fr,ft)
% Given a reference image fr, test image ft and warp coefficients c,
% calculate differences between the reference image and warped test image
% Return E, the sum of the differences squared, and the first (gr) and
% second (H) derivative of E with respect to c
%
% warp coefficients c should define g that is monotonous and 0<=g(x)<=1
% Now adds penalty function
% Seems to be working fine with linear interpolation
% Changed to use quadratic interpolation for g
% Two boundary points of c are not used directly
%  function [E,gr,H]=imwarpcalcb(c,fr,ft)

fr=fr(:)' ;
nr=length(fr) ;
nt=length(ft) ;
nc=length(c) ;
fwt=zeros(1,nr) ;


% Calculate spline coefficients for interpolation of ft
b=cbanal(ft) ;

% Start the interpolation
for i=1:nr,
  % Find gc(i) by interpolation 
   gi=qbinterp(c,2+(i-1)/(nr-1)*(nc-3)) ;
  % Find the value of the transformed image
  fwt(i)=cbinterp(b,gi) ;
end ;  

% Now, the warped test image is in fwt

e=fr-fwt ;
if 0,
  figure(1) ; clf ;
  plot(1:nr,[fr ; fwt]) ; legend('Original','Test') ;
end ;

E=sum(e.^2) ;
gr=zeros(1,nc) ;
H=zeros(nc,nc) ;
dereg=zeros(1,nc) ;

for k=1:nr,
  % Calculate g(k) ;
  gk=qbinterp(c,2+(k-1)/(nr-1)*(nc-3)) ;
  for i=1:nc, 
    % Calculating the derivative 'der=d e(k)/d c(i)'
    der=0 ;
    beta=qbspln((k-1)/(nr-1)*(nc-3)+2-i) ;
    if beta~=0,
%      disp([ 'k= ' num2str(k) ' i=' num2str(i) ' gk=' num2str(gk) ' beta=' num2str(beta)]) ;
      ll=max(floor(gk)-1,1) ;
      for l=ll:min(ll+3,nt),
	der=der-b(l)*cbderiv(gk-l) ;
      end ;
      der=der*beta ;
    end ;  
    dereg(i)=der ;
  end ;
  
  gr=gr+2*dereg*e(k) ;
  H=H+2*dereg'*dereg ;
end ;  

% Now add the penalty function, if desired
if 1,
  alpha=3 ; gamma=0.01 ;
  q=zeros(1,nc-1) ;
  q(1)=exp(alpha*(c(1)-c(3))) ;
  q(nc-1)=exp(alpha*(c(nc-2)-c(nc))) ;
  for i=2:nc-2,
    q(i)=exp(alpha*(c(i)-c(i+1))) ;
  end ;
  
  E=E+gamma*sum(q) ;
  
  gr(1:3)=gr(1:3)+gamma*alpha*[q(1) q(2) q(3)-q(2)-q(1)] ;
  gr(nc-2:nc)=gr(nc-2:nc)+gamma*alpha*[q(nc-2)-q(nc-3)+q(nc-1) -q(nc-2) -q(nc-1)] ;
  H(1:3,1:4)=H(1:3,1:4)+gamma*alpha*alpha*[q(1) 0 -q(1) 0 ; 0 q(2) -q(2) 0 ; ...
        -q(1) -q(2) q(3)+q(2)+q(1) -q(3)] ;
  H(nc-2:nc,nc-3:nc)=H(nc-2:nc,nc-3:nc)+gamma*alpha*alpha*...
      [-q(nc-3) q(nc-2)+q(nc-1)+q(nc-3) -q(nc-2) -q(nc-1) ; 0 -q(nc-2) ...
	q(nc-2) 0 ; 0 -q(nc-1) 0 q(nc-1) ] ;
      
  for i=4:nc-3, 
    gr(i)=gr(i)+gamma*alpha*(q(i)-q(i-1)) ; 
    H(i,i-1:i+1)=H(i,i-1:i+1)+gamma*alpha*alpha*[-q(i-1) q(i-1)+q(i) -q(i)] ;
  end ;
  
end ;

      