function [E,gr,H]=imwarpcalcc(c,fr,ft)
% Given a reference image fr, test image ft and warp coefficients c,
% calculate differences between the reference image and warped test image
% Return E, the sum of the differences squared, and the first (gr) and
% second (H) derivative of E with respect to c
%
% warp coefficients c should define g that is monotonous and 0<=g(x)<=1
% Now adds penalty function
% Seems to be working fine with linear interpolation
% Changed to use cubic interpolation for g

fr=fr(:)' ;
nr=length(fr) ;
nt=length(ft) ;
nc=length(c) ;
fwt=zeros(1,nr) ;


% Calculate spline coefficients for interpolation of ft
b=cbanal(ft) ;

% Start the interpolation
for i=1:nr,
  % Find gc(i) by interpolation 
  % gi=lininterp(c,1,nr,i) ;
  gi=scbinterp(c,1,nr,i) ;
  % Find the value of the transformed image
  fwt(i)=cbinterp(b,gi) ;
end ;  

% Now, the warped test image is in fwt

e=fr-fwt ;
plot(1:nr,[fr ; fwt]) ;

E=sum(e.^2) ;
gr=zeros(1,nc) ;
H=zeros(nc,nc) ;
dereg=zeros(1,nc) ;

for k=1:nr,
  % Calculate g(k) ;
  gk=scbinterp(c,1,nr,k) ;
  for i=1:nc, 
    % Calculating the derivative 'der=d e(k)/d c(i)'
    der=0 ;
    beta=cbspln((k-1)/(nr-1)*(nc-1)+1-i) ;
    if beta~=0,
      ll=max(floor(gk)-1,1) ;
      for l=ll:min(ll+3,nt),
	der=der-b(l)*cbderiv(gk-l)*beta ;
      end ;
    end ;  
    dereg(i)=der ;
  end ;
  
  gr=gr+2*dereg*e(k) ;
  H=H+2*dereg'*dereg ;
end ;  

% Now add the penalty function, if desired
if 1,
  alpha=3 ; gamma=0.01 ;
  for i=1:nc,
    if i>1,
      gexpimoi=gamma*exp(alpha*(c(i-1)-c(i))) ;
      E=E+gexpimoi ;
      gr(i)=gr(i)-alpha*gexpimoi ;
      H(i,i-1)=H(i,i-1)-alpha*alpha*gexpimoi ;
      H(i,i)=H(i,i)+alpha*alpha*gexpimoi ;
    end ;
    if i<nc,
      gexpipoi=gamma*exp(alpha*(c(i)-c(i+1))) ;
      gr(i)=gr(i)+alpha*gexpipoi ;
      H(i,i+1)=H(i,i+1)-alpha*alpha*gexpipoi ;
      H(i,i)=H(i,i)+alpha*alpha*gexpipoi ;
    end ;  
  end ;
end ;

      