function [E,gr,H]=imwarpcalcd(d,fr,ft)
% Given a reference image fr, test image ft and warp coefficients d,
% calculate differences between the reference image and warped test image
% Return E, the sum of the differences squared, and the first (gr) and
% second (H) derivative of E with respect to d
%
% warp coefficients c should define g that is monotonous and 0<=g(x)<=1
% Now adds penalty function
% Seems to be working fine with linear interpolation
% Changed to use quadratic interpolation for g
% Two boundary points of c are not used directly
%  function [E,gr,H]=imwarpcalcb(c,fr,ft)

fr=fr(:)' ;
nr=length(fr) ;
nt=length(ft) ;
nc=length(d) ;
fwt=zeros(1,nr) ;


% Calculate spline coefficients for interpolation of ft
b=cbanal(ft) ;

c=pardtoc(d) ;
% Start the interpolation
for i=1:nr,
  % Find gc(i) by interpolation 
   gi=qbinterp(c,2+(i-1)/(nr-1)*(nc-3)) ;
  % Find the value of the transformed image
  fwt(i)=cbinterp(b,gi) ;
end ;  

% Now, the warped test image is in fwt

e=fr-fwt ;
if 0,
  figure(1) ; clf ;
  plot(1:nr,[fr ; fwt]) ; legend('Original','Test') ;
end ;

E=sum(e.^2) ;
gr=zeros(1,nc) ;
H=zeros(nc,nc) ;
dereg=zeros(1,nc) ;

for k=1:nr,
  % Calculate g(k) ;
  gk=qbinterp(c,2+(k-1)/(nr-1)*(nc-3)) ;
  for i=1:nc, 
    % Calculating the derivative 'der=d e(k)/d c(i)'
    der=0 ;
    beta=qbspln((k-1)/(nr-1)*(nc-3)+2-i) ;
    if beta~=0,
%      disp([ 'k= ' num2str(k) ' i=' num2str(i) ' gk=' num2str(gk) ' beta=' num2str(beta)]) ;
      ll=max(floor(gk)-1,1) ;
      for l=ll:min(ll+3,nt),
	der=der-b(l)*cbderiv(gk-l) ;
      end ;
      der=der*beta ;
    end ;  
    dereg(i)=der ;
  end ;
  dereg=derctoderd(dereg,d) ;
  gr=gr+2*dereg*e(k) ;
  H=H+2*dereg'*dereg ;
end ;  


      