function [c,E]=imwarpopt(fr,ft,nc) ;
% Given a test image ft and reference image fr, find warp coefficients c
% so that the total square difference E between the reference image and
% warped test image is minimal
% nc is the number of nodes interpolating the warping function
%
% Uses Levenberg-Marquardt algorithm inspired by Numerical Recipies
% Does not check for constraints


nr=length(fr) ;
nt=length(ft) ;
c=linspace(1,nr,nc) ;

[Eold,gr,H]=imwarpcalc(c,fr,ft) ;
E=Eold ;
Eold=2*Eold+1 ;
prec=1e-2 ;
lambda=0.0001 ;
i=1 ;

while ((Eold-E)/Eold>1e-4),
    disp([ 'Iteration ' num2str(i) '  criterion=' num2str(E)]) ;
    fwt=imwarp(ft,c,nr) ;
    figure(1) ; clf ;
    plot(1:nr,[fr;ft]) ; legend('Original','Test') ;

  E1=E ;
  dc=-((H+eye(nc)*lambda)\gr')' ;
  % careful adding, taking constraints into account
  % cpdc=addconsmon(c,dc) ;
  cpdc=c+dc ;
  [E,gr1,H1]=imwarpcalc(cpdc,fr,ft) ;
  i=i+1 ;
  if E>E1,
    lambda=10*lambda ; E=E1 ;
    disp('Increasing lambda and trying again') ;
  else
    lambda=lambda/10 ;
    c=cpdc ;
    Eold=E1 ; H=H1 ; gr=gr1 ;
  end ;
end ;
