/* imwarpopt.c The C source file, routine called from meximwarpopt

% Given a test image ft and reference image fr, find warp coefficients c
% so that the total square difference E between the reference image and
% warped test image is minimal
% nc is the number of nodes interpolating the warping function horizontally
% and nd vertically
%
% Uses Levenberg-Marquardt algorithm inspired by Numerical Recipies
% Does not check for constraints but uses penalty function instead
% Now uses linear/quadratic/cubic B-spline interpolation for the warping 
% function g
%
% fr and ft are two dimensional, with the same number of rows. Warping takes
% place across columns
% cin is the starting value for c

  Usage: [c,E]=meximwarpopt(fr,ft,cin,degree)

  Jan Kybic, June 1998

*/

/* Things to set:      ------------------------------------------ */

#define PREC (1e-4)   /* relative precision */
#define DELTA (1e-10) /* absolute precision */
#define LAMBDAK (10.0)  /* lambda multiplicator */

/* end of things to set ----------------------------------------- */

#include <math.h>
#ifndef MAKING_MEX
#include <stdio.h>
#include <stdlib.h>
#define mexErrMsgTxt(x) fprintf(stderr,x) 
#define NAME imwarpcalc
#define SAFEMALLOC(ptr,a,b) \
 if ((ptr=(void *)malloc((a)*(b)*sizeof(double)))==NULL) \
    mexErrMsgTxt(NAME ": could not malloc") ;

#define SAFEMALLOCINT(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(int)))==NULL) \
    mexErrMsgTxt(NAME ": could not malloc") ;

#define SAFEFREE(x) mxFree(x)
#endif /* ifndef MAKING_MEX */

/* c is the 2D matrix of coefficients decribing the warping function
   by using splines of degree `degree', with nc columns
   and nd rows. fr is the reference image with nr columns and ny rows,
   ft is the test image with nt columns and ny rows
   The criterion is returned in E, the optimal parameters in c.
   All matrices are stored columnwise, i.e. in the Matlab/Fortran
   fashion. */

void imwarpopt(double *c,double *fr,double *ft,
		   double *E,int nr,int nt,int nc,int nd,int ny,int degree) 
{
  double *gr,*H,*ftb ;
  int i,j ;

  /* allocate temporary memory needed */
  SAFEMALLOC(gr,nc,nd) ;  SAFEMALLOC(H,nc*nd,nc*nd) ;

  /* find cubic spline coefficients for the test image */




  SAFEFREE(gr) ; SAFEFREE(H) ;
}

