function [c,E]=imwarpoptbbddi(fr,ft,nc,nd,c) ;
% Given a test image ft and reference image fr, find warp coefficients c
% so that the total square difference E between the reference image and
% warped test image is minimal
% nc is the number of nodes interpolating the warping function horizontally
% and nd vertically
%
% Uses Levenberg-Marquardt algorithm inspired by Numerical Recipies
% No penalty function, going out of the permissible space is regarded as
% failure 
% Now uses quadratic B-spline interpolation for the warping function g
%
% fr and ft are two dimensional, with the same number of rows. Warping takes
% place across columns
%
% c is the initial estimate of the warping coefficients
%
% Usage: [c,E]=imwarpoptbbddi(fr,ft,nc,nd,c) ;



[ny nr]=size(fr) ;
[nyt nt]=size(ft) ;

if ny~=nyt, error('fr and ft must have the same number of rows') ; end ;

if nc<4 & nd<4, error('nc,nd must be at least 4.') ; end ;

ec=reshape(c,nd,nc) ;
e=max(max(abs(ec)))*1e-6 ;
for i=1:nd,
  if ec(i,1)>ec(i,3), ec(i,1)=ec(i,3)-e ; end ;
  for j=3:nc-1,
    if ec(i,j)<ec(i,j-1),  ec(i,j)=ec(i,j-1)+e ; end ;
  end ;
  if ec(i,nc)<ec(i,nc-2), ec(i,nc)=ec(i,nc-2)+e ; end ;
end ;

c=reshape(ec,1,nc*nd) ;



% disp( 'Image analyse started') ;
ftb=imcbanal(ft) ;
% disp( 'Image analyse finished') ;

[Eold,gr,H]=meximwarpcalcbbdd(c,fr,ft,ftb,nc) ;
E=Eold ;
Eold=2*Eold+1 ;
prec=1e-4   ; % or 1e-2
lambda=0.1 ; % or 1e-4
i=1 ; j=0 ;

% while ((Eold-E)/Eold>prec | j<3),
while ((Eold-E)/Eold>prec),
%    disp([ 'Iteration ' num2str(i) '  criterion=' num2str(E)]) ;

  E1=E ;
  M=H+eye(nc*nd)*lambda ; rc=rcond(M) ;
  if rc>1e-10,
    dc=-gr/M' ;
    cpdc=c+dc ;
    [E,gr1,H1]=meximwarpcalcbbdd(cpdc,fr,ft,ftb,nc) ;
    i=i+1 ;
  else
    disp('Linear equation set close to singular.') ;
  end ;
  if E<0,
    disp('Out of permissible space') ;
    i=i-1 ;
  end ;
  
  if E>E1 | rc<=1e-10 | E<0,
    lambda=10*lambda ; E=E1 ;
%    lambda=2*lambda ; E=E1 ;
    disp(['Increasing lambda and trying again. lambda=' num2str(lambda)]) ;
    j=0 ;
  else
    lambda=lambda/10 ;
%    lambda=lambda/2 ; 
    j=j+1 ;
    c=cpdc ;
    Eold=E1 ; H=H1 ; gr=gr1 ;
  end ;
end ;
  
disp([ 'Iteration ' num2str(i) '  criterion=' num2str(E)]) ;