function [c,E]=imwarpoptbdd(fr,ft,nc,nd) ;
% Given a test image ft and reference image fr, find warp coefficients c
% so that the total square difference E between the reference image and
% warped test image is minimal
% nc is the number of nodes interpolating the warping function horizontally
% and nd vertically
%
% Uses Levenberg-Marquardt algorithm inspired by Numerical Recipies
% Does not check for constraints but uses penalty function instead
% Now uses quadratic B-spline interpolation for the warping function g
%
% fr and ft are two dimensional, with the same number of rows. Warping takes
% place across columns


[ny nr]=size(fr) ;
[nyt nt]=size(ft) ;

if ny~=nyt, error('fr and ft must have the same number of rows') ; end ;

if nc<4 & nd<4, error('nc,nd must be at least 4.') ; end ;

% Preparing the start conditions

c=[0 linspace(1,nr,nc-2) 0];
c(1)=2*c(2)-c(3) ; c(nc)=2*c(nc-1)-c(nc-2) ;
row=qbanal(c) ;
c=zeros(nd,nc) ;
for i=1:nd, c(i,:)=row ; end ;
c=reshape(c,1,nc*nd) ;


ftb=imcbanal(ft) ;

[Eold,gr,H]=meximwarpcalcbdd(c,fr,ft,ftb,nc) ;
E=Eold ;
Eold=2*Eold+1 ;
prec=1e-2   ;
lambda=0.01 ;
i=1 ; j=0 ;

while ((Eold-E)/Eold>prec | j<3), 
  % previously while ((Eold-E)/Eold>prec | j<3),
    disp([ 'Iteration ' num2str(i) '  criterion=' num2str(E)]) ;

  E1=E ;
  M=H+eye(nc*nd)*lambda ; rc=rcond(M) ;
  if rc>1e-10,
    dc=-gr/M' ;
    cpdc=c+dc ;
    [E,gr1,H1]=meximwarpcalcbdd(cpdc,fr,ft,ftb,nc) ;
    % disp(['Returned E' num2str(E)]) ;
    % E
    i=i+1 ;
  else
    disp('Linear equation set close to singular.') ;
  end ;
  if E>E1 | rc<=1e-10,
    lambda=10*lambda ; E=E1 ;
    disp(['Increasing lambda and trying again lambda=' num2str(lambda)]) ;
    j=0 ;
  else
    lambda=lambda/2 ;
    c=cpdc ;
    Eold=E1 ; H=H1 ; gr=gr1 ;
    j=j+1 ;
  end ;
end ;

disp([ 'Iteration ' num2str(i) '  criterion=' num2str(E)]) ;