function [c,E]=imwarpoptbtd(fr,ft,nc) ;
% Given a test image ft and reference image fr, find warp coefficients c
% so that the total square difference E between the reference image and
% warped test image is minimal
% nc is the number of nodes interpolating the warping function
%
% Uses Levenberg-Marquardt algorithm inspired by Numerical Recipies
% Does not check for constraints but uses penalty function instead
% Now uses quadratic B-spline interpolation for the warping function g
%
% fr and ft are two dimensional, with the same number of rows. Warping takes
% place across columns


[ny nr]=size(fr) ;
[nyt nt]=size(ft) ;

if ny~=nyt, error('fr and ft must have the same number of rows') ; end ;

if nc<4, error('nc must be at least 4.') ; end ;

% Preparing the start conditions
c=[0 linspace(1,nr,nc-2) 0];
c(1)=2*c(2)-c(3) ; c(nc)=2*c(nc-1)-c(nc-2) ;
c=qbanal(c) ;

ftb=imcbanal(ft) ;

[Eold,gr,H]=meximwarpcalcbtd(c,fr,ft,ftb) ;
E=Eold ;
Eold=2*Eold+1 ;
prec=1e-4   ;
lambda=0.0001 ;
i=1 ;

while ((Eold-E)/Eold>prec),
    disp([ 'Iteration ' num2str(i) '  criterion=' num2str(E)]) ;

  E1=E ;
  dc=-((H+eye(nc)*lambda)\gr')' ;
  cpdc=c+dc ;
  [E,gr1,H1]=meximwarpcalcbtd(cpdc,fr,ft,ftb) ;
  i=i+1 ;
  if E>E1,
    lambda=10*lambda ; E=E1 ;
    disp('Increasing lambda and trying again') ;
  else
    lambda=lambda/10 ;
    c=cpdc ;
    Eold=E1 ; H=H1 ; gr=gr1 ;
  end ;
end ;
  
disp([ 'Iteration ' num2str(i) '  criterion=' num2str(E)]) ;