function [c,E]=imwarpoptdcf(fr,ft,nc) ;
% Given a test image ft and reference image fr, find warp coefficients c
% so that the total square difference E between the reference image and
% warped test image is minimal
% nc is the number of nodes interpolating the warping function
%
% Uses Levenberg-Marquardt algorithm inspired by Numerical Recipies
% Does not check for constraints but uses penalty function instead
% Now uses quadratic B-spline interpolation for the warping function g
% d-parametrisation to get rid of the constraints

nr=length(fr) ;
nt=length(ft) ;

if nc<4, error('nc must be at least 4.') ; end ;

c=[0 linspace(1,nr,nc-2) 0];
c(1)=2*c(2)-c(3) ; c(nc)=2*c(nc-1)-c(nc-2) ;
c=qbanal(c) ;
d=parctod(c) ;
ftb=cbanal(ft) ;

[Eold,gr,H]=imwarpcalcd(d,fr,ft) ;
E=Eold ;
Eold=2*Eold+1 ;
prec=1e-4 ;
lambda=0.0001 ;
i=1 ;


while ((Eold-E)/Eold>prec),
    disp([ 'Iteration ' num2str(i) '  criterion=' num2str(E)]) ;

  E1=E ;
  if 1,
    dd=-((H+eye(nc)*lambda)\gr')' ;
    else
      % svd implementation - more robust but apparently unnecessary
      [u,s,v]=svd(H+eye(nc)*lambda) ;
      for j=1:nc, 
	if abs(s(j,j))>1e-6, 
	  s(j,j)=1/s(j,j) ;
	else
	  s(j,j)=0 ;
	end ;
      end ;
      dd=-gr*v*s*u' ;
    end ;
  dpdd=d+dd ;
  %  [E,gr1,H1]=meximwarpcalcb(cpdc,fr,ft) ;
  [E,gr1,H1]=imwarpcalcd(dpdd,fr,ft) ;
  i=i+1 ;
  if E>E1,
    lambda=10*lambda ; E=E1 ;
    disp('Increasing lambda and trying again') ;
  else
    lambda=lambda/10 ;
    d=dpdd ;
    Eold=E1 ; H=H1 ; gr=gr1 ;
  end ;
end ;
  
disp([ 'Iteration ' num2str(i) '  criterion=' num2str(E)]) ;

c=pardtoc(d) ;
