/*
  meximwarpbdd    MEX file source

  Given a 2D function sampled at discrete points given by its cubic
  B-spline interpolation coefficients f and a 2D warping function
  represented by quadratic B-spline interpolation coefficients c at discrete 
  equidistant points (plus one at each side), 
  return a new image y of n columns. The number of rows stays unchanged
 
  Currently uses cubic B-spline interpolation for 
  and quadratic B-spline interpolation for f. The interpolation for g is 
  carried  out at region corresponding to c(2)..c(n-1) of each row/column.
  The warping matrix is stored in a vector columnwise, the number
  of columns given by nc

  Calling: y=meximwarpbdd(f,c,n,nc)


  Jan Kybic, April 1997

*/

/* #ifdef __GNUC__ 
 #ifndef linux 
 #define linux 
 #endif 
 #endif */

#ifdef linux
#define myCreateMatrix(a,b) mxCreateFull(a,b,REAL)
#define myAlloc(a) mxCalloc(a,1)
#else /* hp version */
#define myCreateMatrix(a,b) mxCreateDoubleMatrix(a,b,mxREAL)
#define myAlloc(a) mxMalloc(a)
#endif

#include <string.h>
#include <math.h>
#include "mex.h"

/* Input Arguments */

#define	F_IN	prhs[0]
#define	C_IN	prhs[1]
#define	N_IN	prhs[2]
#define NC_IN   prhs[3]

/* Output Arguments */

#define	Y_OUT	plhs[0]


#define max(a,b) ((a>b) ? (a) : (b))

void imwarpbdd(double *yp,double *fp,double *cp,int n,int nf,int nc,int nd,
	       int ny) ;

#ifdef linux
void mexFunction(
                 int nlhs, Matrix *plhs[],
                 int nrhs, Matrix *prhs[]
		 )
#else
void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
#endif
{ double *yp,*fp,*cp ;
  int i,n,nf,nc,nd,nct,ny ;

  /* Check for proper number of arguments */
  
  if (nrhs != 4) {
    mexErrMsgTxt("IMWARPBDD requires four input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt("IMWARPBDD requires one output argument.");
  }
  
  for (i=0;i<nrhs;i++)
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(
             "IWARPBDD needs all input arguments to be real.");
     } 
  
  ny=mxGetM(F_IN) ; nf=mxGetN(F_IN) ;
  nc=(int)(*(mxGetPr(NC_IN))) ;
  nct=max(mxGetM(C_IN),mxGetN(C_IN)) ;
  nd=nct/nc ;  

  yp = mxGetPr(N_IN); fp=mxGetPr(F_IN) ; cp = mxGetPr(C_IN); 
  n=(int)(*yp) ;

  Y_OUT = myCreateMatrix(ny,n);
  yp=mxGetPr(Y_OUT) ;

  /*  %%%%% here the calculation begins %%%%%% */
  imwarpbdd(yp,fp,cp,n,nf,nc,nd,ny) ;
  return;
}

#define SAFEMALLOC(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(double)))==NULL) \
    mexErrMsgTxt("IMWARPBDD: could not malloc") ;

#define SAFEMALLOCINT(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(int)))==NULL) \
    mexErrMsgTxt("IMWARPBDD: could not malloc") ;

#define MAKING_MEX

#include "bsplinetools.c"

