/*
  meximwarpbtd    The MEX version of imwarpbtd.m 

  Given a 2D function sampled at discrete points given by its cubic
  B-spline interpolation coefficients f and a warping function
  represented by quadratic B-spline interpolation coefficients c at discrete 
  equidistant points (plus one at each side), 
  return a new image y of n columns. The number of rows stays unchanged
 
  Currently uses cubic B-spline interpolation for 
  and quadratic B-spline interpolation for f. The interpolation for g is 
  carried  out at region corresponding to c(2)..c(n-1) 

  Calling: y=meximwarpbtd(f,c,n)


  Jan Kybic, April 1997

*/

/* #ifdef __GNUC__ 
 #ifndef linux 
 #define linux 
 #endif 
 #endif */

#ifdef linux
#define myCreateMatrix(a,b) mxCreateFull(a,b,REAL)
#define myAlloc(a) mxCalloc(a,1)
#else /* hp version */
#define myCreateMatrix(a,b) mxCreateDoubleMatrix(a,b,mxREAL)
#define myAlloc(a) mxMalloc(a)
#endif

#include <string.h>
#include <math.h>
#include "mex.h"

/* Input Arguments */

#define	F_IN	prhs[0]
#define	C_IN	prhs[1]
#define	N_IN	prhs[2]

/* Output Arguments */

#define	Y_OUT	plhs[0]


#define max(a,b) ((a>b) ? (a) : (b))

void imwarpbtd(double *yp,double *fp,double *cp,int n,int nf,int nc,int ny) ;

#ifdef linux
void mexFunction(
                 int nlhs, Matrix *plhs[],
                 int nrhs, Matrix *prhs[]
		 )
#else
void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
#endif
{ double *yp,*fp,*cp ;
  int i,n,nf,nc,ny ;

  /* Check for proper number of arguments */
  
  if (nrhs != 3) {
    mexErrMsgTxt("IMWARPBTD requires three input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt("IMWARPBTD requires one output argument.");
  }
  
  for (i=0;i<nrhs;i++)
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(
             "IWARPBTD needs all input arguments to be real.");
     } 
  
  ny=mxGetM(F_IN) ; nf=mxGetN(F_IN) ;
  nc=max(mxGetM(C_IN),mxGetN(C_IN)) ;

  yp = mxGetPr(N_IN); fp=mxGetPr(F_IN) ; cp = mxGetPr(C_IN); 
  n=(int)(*yp) ;

  Y_OUT = myCreateMatrix(ny,n);
  yp=mxGetPr(Y_OUT) ;

  /*  mexPrintf("IMWARPBTD called with nf=%d nc=%d ny=%d n=%d\n",
   nf,nc,ny,n) ;
  */

  /*  %%%%% here the calculation begins %%%%%% */
  imwarpbtd(yp,fp,cp,n,nf,nc,ny) ;
  return;
}

#define SAFEMALLOC(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(double)))==NULL) \
    mexErrMsgTxt("IMWARPBTD: could not malloc") ;

#define SAFEMALLOCINT(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(int)))==NULL) \
    mexErrMsgTxt("IMWARPBTD: could not malloc") ;

#define MAKING_MEX

#include "bsplinetools.c"

