function [c,E]=pimwarpoptbdd(frp,ftp,nc,nd) ;
% Given a test image ft and reference image fr, find warp coefficients c
% so that the total square difference E between the reference image and
% warped test image is minimal
% nc is the number of nodes interpolating the warping function horizontally
% and nd vertically
%
% Uses Levenberg-Marquardt algorithm inspired by Numerical Recipies
% Does not check for constraints but uses penalty function instead
% Now uses quadratic B-spline interpolation for the warping function g
% Multiresolution pyramid approach
%
% fr and ft are two dimensional, with the same number of rows. Warping takes
% place across columns

fr=frp{1} ; ft=ftp{1} ;
% Calculate the pyramides
nl=min(log(length(fr)),log(length(ft)))/log(2) ;
nl=floor(max(nl,0)) ;
%disp('Calculating pyramides') ;
%figure(1) ;
%frp=pyramid(fr,nl) ; 
%figure(2) ;
%ftp=pyramid(ft,nl) ;
nca=4 ; nda=4 ;

[ony onr]=size(fr) ; 
[fullny fullnr]=size(fr) ;

for i=nl:-1:1,
  frc=frp{i} ; ftc=ftp{i} ;
  if i~=nl, 
    oldnr=nr ; 
  end ;
  ny=size(frc,1) ; nr=size(frc,2) ; nt=size(ftc,2) ;
  disp([ 'Level ' num2str(i) ' rows=' num2str(ny) ' colref=' num2str(nr) ...
	' coltest=' num2str(nt) ' nca=' num2str(nca) ' nda=' num2str(nda)]) ;

  subplot(221) ; imagesc(frc) ; title('reference') ; colorbar ; 
  subplot(222) ; imagesc(ftc) ; title('test') ; colorbar ;
  colormap(jet) ;
  pause(0.01) ;  
  
  if i==nl, % The first time -> initialize c
    c=[0 linspace(1,nr,nca-2) 0];
    c(1)=2*c(2)-c(3) ; c(nca)=2*c(nca-1)-c(nca-2) ;
    row=qbanal(c) ;
    c=zeros(nda,nca) ;
    for i=1:nda, c(i,:)=row ; end ;
    c=imqbanal(c')' ;
    c=reshape(c,1,nca*nda) ;
  else
    % image dimension was increased, change c appropriately
    c=expandc(c,nca,oldnr,nr,nca,nda) ;
  end ;
  
  
  % Optimize 
  [c,E]=imwarpoptbddi(frc,ftc,nca,nda,c) ;
%  [c,E]=imwarpoptbbddi(frc,ftc,nca,nda,c) ;
  
  imeb=imcbanal(ftc) ;
  imh=meximwarpbdd(imeb,c,nr,nca) ;
  subplot(223) ; imagesc(imh) ; title('warped') ; colorbar ; grid on ;
  subplot(224) ; imagesc(imh-frc) ; title('difference') ; colorbar ; grid on ;
  % disp('Press a key') ; 
  pause(0.01) ;
  
  % Increase nca, nda, if appropriate
  oldnca=nca ; oldnda=nda ;
%  nca=4+(nc-4)*(nr/fullnr)^(1/4) ; nda=4+(nd-4)*(ny/fullny)^(1/4) ;
  nca=min(2+nr/4,nc) ;   nda=min(2+nr/4,nd) ;  
  nca=ceil(max(4,nca)) ;  nda=ceil(max(4,nda)) ;
  if nca~=oldnca | nda~=oldnda,
    disp([ 'Expanding parameter space: nca=' num2str(nca) ...
	  ' nda=' num2str(nda)]) ;
    c=expandc(c,oldnca,nr,nr,nca,nda) ;
    % Optimize 
    [c,E]=imwarpoptbddi(frc,ftc,nca,nda,c) ;
%    [c,E]=imwarpoptbbddi(frc,ftc,nca,nda,c) ;
  
    imh=meximwarpbdd(imeb,c,nr,nca) ;
    subplot(223) ; imagesc(imh) ; title('warped') ; colorbar ; grid on ;
    subplot(224) ; imagesc(imh-frc) ; title('difference') ; colorbar ; grid on ;
%    disp('Press a key') ; 
  pause(0.01) ;
  end ;  
end ;

% move into permissible space

ec=reshape(c,nd,nc) ;
e=max(max(abs(ec)))*1e-6 ;
for i=1:nd,
  if ec(i,1)>ec(i,3), ec(i,1)=ec(i,3)-e ; end ;
  for j=3:nc-1,
    if ec(i,j)<ec(i,j-1),  ec(i,j)=ec(i,j-1)+e ; end ;
  end ;
  if ec(i,nc)<ec(i,nc-2), ec(i,nc)=ec(i,nc-2)+e ; end ;
end ;

c=reshape(ec,1,nc*nd) ;

% no use
% [c,E]=imwarpoptbbddi(frc,ftc,nca,nda,c) ;

 