function [c,nca]=regimlocn(refim,testim,nc,nd,order) ;
% Given reference image r and a test image t, find matrix (although returned
% as row vector) c of nd rows and nc columns of quadratic B-spline
% coefficients describing the transformation function
%
% For the moment, images are expected to be the same size. nca contains the
% number of columns for c when converted to matrix form nca<=nc
%
% Unlike regim, uses local normalization instead of High-pass filtering
%
% Usage: c=regimlocn(r,t,nc,nd,order) ;

cmsf=64 ;

% so that the coarsest level has size of 32-64 pixels
nl=min(log(length(refim)),log(length(testim)))/log(2)-4 ;
nl=floor(max(nl,0)) ;

filtsize=20 ;

frefim=locnorm(refim,filtsize) ;
ftestim=locnorm(testim,filtsize) ;

figure(2) ;
subplot(221) ; imagesc(refim) ; title('reference') ;
subplot(222) ; imagesc(testim) ; title('test') ;
subplot(223) ; imagesc(frefim) ; title('filt. ref.') ;
subplot(224) ; imagesc(ftestim) ; title('filt. test') ;
figure(1) ;

ctime=cputime ;
refp=pyramid2(frefim,nl) ;
testp=pyramid2(ftestim,nl) ;
disp([ 'Pyramids took ' num2str(cputime-ctime) ' s' ]) ;

[c,E,nca]=pimwarpopt(refp,testp,nc,nd,order) ;

