/*
  meximwarpdc    MEX file source

  Given a 2D function sampled at discrete points given by its cubic
  B-spline interpolation coefficients f and a 2D->2D warping function
  represented by cubic B-spline interpolation
  coefficients c at discrete equidistant points (plus one at each
  side), return a new image y of nx columns and ny rows. 
 
  Currently uses cubic B-spline interpolation for the images.

  Calling: y=meximwarpdc(f,c,nc,nx,ny)


  Jan Kybic, October 1998

*/

/* #ifdef __GNUC__ 
 #ifndef linux 
 #define linux 
 #endif 
 #endif */

#define myCreateMatrix(a,b) mxCreateDoubleMatrix(a,b,mxREAL)
#define myAlloc(a) mxMalloc(a)

#include <string.h>
#include <math.h>
#include "mex.h"

/* Input Arguments */

#define	F_IN	prhs[0]
#define	C_IN	prhs[1]
#define	NC_IN	prhs[2]
#define NX_IN   prhs[3]
#define NY_IN   prhs[4]


/* Output Arguments */

#define	Y_OUT	plhs[0]

#define NAME "MEXIMWARPD"

#define max(a,b) ((a>b) ? (a) : (b))

void imwarpdcdd(double *yp,double *fp,double *cp,int nfx,int nfy,int ntx, 
		int nty, int nc,int nd) ;

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ double *yp,*fp,*cp ;
  int i,nfx,nfy,ntx,nty,nc,nd,nct ;

  /* Check for proper number of arguments */
  
  if (nrhs != 5) {
    mexErrMsgTxt(NAME " requires five input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }
  
  for (i=0;i<nrhs;i++)
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(
             NAME " needs all input arguments to be real.");
     } 
  
  nfy=mxGetM(F_IN) ; nfx=mxGetN(F_IN) ;
  nc=(int)(*(mxGetPr(NC_IN))) ;
  nct=max(mxGetM(C_IN),mxGetN(C_IN)) ;
  nd=nct/nc/2 ;  
  ntx=(int)(*(mxGetPr(NX_IN))) ;
  nty=(int)(*(mxGetPr(NY_IN))) ;

  fp=mxGetPr(F_IN) ; cp = mxGetPr(C_IN); 

  Y_OUT = myCreateMatrix(nty,ntx);
  yp=mxGetPr(Y_OUT) ;

  /*  %%%%% here the calculation begins %%%%%% */
#if 0
  mexPrintf("nfx=%d nfy=%d ntx=%d nty=%d nc=%d nd=%d\n",nfx,nfy,ntx,nty,nc,nd) ;
#endif
  imwarpdcdd(yp,fp,cp,nfx,nfy,ntx,nty,nc,nd) ; 
  return;
}

#define SAFEMALLOC(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(double)))==NULL) \
    mexErrMsgTxt(NAME ": could not malloc") ;

#define SAFEMALLOCINT(ptr,a,b) \
 if ((ptr=(void *)myAlloc((a)*(b)*sizeof(int)))==NULL) \
    mexErrMsgTxt(NAME ": could not malloc") ;

#define MAKING_MEX

#include "bsplinetools.c"

