function dAd=szcalcdad(d,grx,gry,ux,uy,m,t) ;
% Part of the implementation of the Szelinski-Coughlan registration algorithm
%
% Jan Kybic, February 1999
%
% Usage: dAd=szcalcdad(d,grx,gry,ux,uy,m,t) ;

[ny,nx]=size(grx) ;
nsy=floor((ny-2)/m)+1 ;
nsx=floor((nx-2)/m)+1 ;
[w,lbas,hbas]=szbas(0,0,t) ; % get the support size

dAd=0 ;
% k,l is the row, resp. column of the reference image
for k=1:ny,
  for l=1:nx,
    % fetch the coordinates in the test image
    uxc=ux(k,l) ; uyc=uy(k,l) ;
    if uxc>=2 & uxc<=nx-1 & uyc>=2 & uyc<=ny-1,
      % ignore points outside  the image
      % leave margin of 1 to allow for (bilinear interpolation)
      rx=(l-1.5)/m+1 ; % coordinates in the control grid space
      ry=(k-1.5)/m+1 ;
      if rx>=1 & rx<=nsx & ry>=1 & ry<=nsy, % ignore points outside  the grid
	% Boundaries of influential control points
	lrx=max(1,ceil(rx-hbas)) ;
	hrx=min(nsx,floor(rx-lbas)) ;
	lry=max(1,ceil(ry-hbas)) ;
	hry=min(nsy,floor(ry-lbas)) ;
	gx=grx(k,l) ;
	gy=gry(k,l) ;

	% For all influential parameters
	du=0 ; dv=0 ;
	for ll=lrx:hrx,
	  for kk=lry:hry,
	    ind=2*((ll-1)*nsy+kk)-1 ;
	    wij=szbas(rx-ll,ry-kk,t) ;
            du=du+wij*d(ind) ;
	    dv=dv+wij*d(ind+1) ;
	  end ; % for ll
	end ;  % for kk
	dAd=dAd+(gx*du+gy*dv)^2 ;
	
      end ; % if rx>=1 ...
    end ; % if uxc<1 ...   
  end ;	% for l 
end ; % for k
