function [Ebest,s]=szopt(fr,ft,m,t,s)
% Part of the implementation of the Szelinski-Coughlan registration algorithm
% Given two images, reference (fr) and test (ft) as well as the 
% subsampling factor m and basis function type t, it calculates
% the optimal control grid s. Return the SSD criterion (E)
%
% Jan Kybic, February 1999
%
% Usage: [E,s]=szopt(fr,ft,m,t)

[ny,nx]=size(fr) ;    % image size
nsx=floor((nx-2)/m)+1 ; % size of the control grid
nsy=floor((ny-2)/m)+1 ;
Ns=nsx*nsy ; % Number of parameter pairs

% initialize s
if nargin==4,
  sx=zeros(nsy,nsx) ;
  s=szpack(sx,sx) ;
elseif nargin~=5,
  error('Wrong number of input arguments') ;
end ;

tracking=1 ;

% lambda=0.001 ; % according to the article
lambda=10000 ; % conservative choice
prec=1e-3 ; % relative precision
delta=1e-40 ; % absolute precision
miniter=9 ; % minimum/maximum number of iterations
maxiter=9 ;

iter=1 ;
fcnt=0 ;
[Ebest,g,H,grx,gry,ux,uy]=szcrit(s,fr,ft,m,t) ;
Eold=Ebest ; Enew=Ebest ;
if tracking,
  Ehistory=Ebest ;
end ;



disp([ 'Iteration ' num2str(iter) ', criterion E=' num2str(Ebest) ]) ;

while (iter<maxiter & (abs(Eold-Enew)>abs(Ebest)*prec+delta | iter<miniter)), 
  % Calculate Binv and d
  Binv=zeros(2,2) ;
  d=zeros(2*Ns,1) ;
  for i=1:2:2*Ns,
    Binv=inv(lambda*eye(2)+H(i:i+1,1:2)) ;
    d(i:i+1)=-Binv*g(i:i+1) ;
  end ;

  % calculate optimal step size alpha
  % dAd=szcalcdad(d,grx,gry,ux,uy,m,t) ;
  % alpha=(d'*g)/dAd  ;
  % alpha=-(d'*g)/(dAd+d'*d*lambda*lambda) ; % My correction
  % if (isnan(alpha)) alpha=0.01 ; end ;
  % disp([ 'Step size' num2str(sqrt(alpha*alpha*d'*d)) ]) ;
  % d=g ; alpha=1/lambda ;
  alpha=1 ;
  st=s+alpha*d ;
  Eold=Enew ;
  [Enew,g1,H1,grx1,gry1,ux1,uy1]=szcrit(st,fr,ft,m,t) ;
  iter=iter+1 ;
  disp([ 'Iteration ' num2str(iter) ', criterion E=' num2str(Enew) ', best E=' ...
	num2str(min([Enew Ebest])) ]) ;
  if Enew>Ebest,
    lambda=10*lambda ; 
    disp(['Increasing lambda and trying again lambda=' num2str(lambda)]) ;
  else 
    lambda=lambda/10 ;
    s=st ;
    Ebest=Enew ; g=g1 ; H=H1 ;
    grx=grx1 ; gry=gry1 ; ux=ux1 ; uy=uy1 ;
  end ;

  if tracking,
    Ehistory=[ Ehistory Ebest ] ;
  end ;

end ;

if tracking,
%  eval([ 'save szopt' num2str(t) 'Ehistory Ehistory' ]) ;
end ;

    
  



    