function [Ebest,s]=szoptgr(fr,ft,m,t,s)
% Part of the implementation of the Szelinski-Coughlan registration algorithm
% Given two images, reference (fr) and test (ft) as well as the 
% subsampling factor m and basis function type t, it calculates
% the optimal control grid s. Return the SSD criterion (E)
%
% This version implements straight gradient descent algorithm
%
% Jan Kybic, February 1999
%
% Usage: [E,s]=szopt(fr,ft,m,t)

[ny,nx]=size(fr) ;    % image size
nsx=floor((nx-2)/m)+1 ; % size of the control grid
nsy=floor((ny-2)/m)+1 ;
Ns=nsx*nsy ; % Number of parameter pairs

% initialize s
if nargin==4,
  sx=zeros(nsy,nsx) ;
  s=szpack(sx,sx) ;
elseif nargin~=5,
  error('Wrong number of input arguments') ;
end ;

tracking=1 ;

% lambda=0.001 ; % according to the article
lambda=1 ; % conservative choice
prec=1e-40 ;
delta=1e-10 ;
iter=1 ;
fcnt=0 ;
[Ebest,g,H,grx,gry,ux,uy]=szcrit(s,fr,ft,m,t) ;
Eold=Ebest ; Enew=Ebest ;
if tracking,
  Ehistory=Ebest ;
end ;

disp([ 'Iteration ' num2str(iter) ', criterion E=' num2str(Ebest) ]) ;

while (abs(Eold-Enew)>abs(Ebest)*prec+delta | iter<=1), 
  st=s-g/lambda ;
  Eold=Enew ;
  [Enew,g1,H1,grx1,gry1,ux1,uy1]=szcrit(st,fr,ft,m,t) ;
  iter=iter+1 ;
  disp([ 'Iteration ' num2str(iter) ', criterion E=' num2str(Enew) ', best E=' ...
	num2str(min([Enew Ebest])) ]) ;
  if Enew>Ebest,
    lambda=10*lambda ; 
    disp(['Increasing lambda and trying again lambda=' num2str(lambda)]) ;
  else 
    lambda=lambda/10 ;
    s=st ;
    Ebest=Enew ; g=g1 ; H=H1 ;
    grx=grx1 ; gry=gry1 ; ux=ux1 ; uy=uy1 ;
  end ;

  if tracking,
    Ehistory=[ Ehistory Ebest ] ;
  end ;

end ;

if tracking,
  eval([ 'save szoptgr' num2str(t) 'Ehistory Ehistory' ]) ;
end ;


  



    