function [E,s]=szpyr(fr,ft,m,t,l)
% Part of the implementation of the Szelinski-Coughlan registration algorithm
% Given two images, reference (fr) and test (ft) as well as the 
% subsampling factor m, basis function type t and the number of pyramid
% levels, it calculates
% the optimal control grid s. Return the SSD criterion (E)
%
% Jan Kybic, February 1999
%
% Usage: [E,s]=szpyr(fr,ft,m,t,l)

frp=gauspyr(fr,l-1) ;
ftp=gauspyr(ft,l-1) ;


for level=l:-1:1,
  fra=frp{level} ; fta=ftp{level} ;
  if level==l,
    [E,s]=szopt(fra,fta,m,t) ;
  else
    [E,s]=szopt(fra,fta,m,t,s) ;
  end ;
  disp([ 'Optimization at level ' num2str(level) ' E=' num2str(E) ] ) ;
  if level>1,
    % Here we have to double the control grid
    [ny,nx]=size(fra) ;    % image size
    nsx=floor((nx-2)/m)+1 ; % size of the control grid
    nsy=floor((ny-2)/m)+1 ;
    [sx,sy]=szunpack(s,nsx,nsy) ;
    % Calculate the new grid size
    [nny,nnx]=size(frp{level-1}) ;    % image size
    nnsx=floor((nnx-2)/m)+1 ; % size of the control grid
    nnsy=floor((nny-2)/m)+1 ;
    % Interpolate and pack
    sx=2*szexp(sx,nnsy,nnsx) ;
    sy=2*szexp(sy,nnsy,nnsx) ;
    s=szpack(sx,sy) ;
  end ;
end ;



    