/* This file reads an Analyze(TM) file */
/* for the moment, just the signed_short integer datatype */
/* J.K. 8.10.1998 */

/* info from "http://www.mayo.edu/bir/analyze/AnalyzeFileInfo.html#FileFormat" */


/* usage: X=mexreadanalyze(filename) 
 - where filename does not contain the extension, `.hdr' and `.img' is added
automatically */

#define DT_NONE                        0 
#define DT_UNKNOWN               0      /*Unknown data type*/ 
#define DT_BINARY                    1      /*Binary (1 bit per voxel)*/ 
#define DT_UNSIGNED_CHAR     2      /*Unsigned character (8 bits per voxel)*/ 
#define DT_SIGNED_SHORT        4      /*Signed short (16 bits per voxel)*/ 
#define DT_SIGNED_INT             8      /*Signed integer (32 bits per voxel)*/ 
#define DT_FLOAT                     16     /*Floating point (32 bits per voxel)*/ 
#define DT_COMPLEX                32     /*Complex (64 bits per voxel; 2 floating point numbers) 
					   #define DT_DOUBLE                   64     /*Double precision (64 bits per voxel)*/ 
#define DT_RGB                         128    /* */
#define DT_ALL                         255    /* */


struct header_key       /* header key   */ 
       {                                /* off + size      */
       int sizeof_hdr;                   /* 0 + 4           */
       char data_type[10];              /* 4 + 10          */
       char db_name[18];                /* 14 + 18         */
       int extents;                     /* 32 + 4          */
       short int session_error;         /* 36 + 2          */
       char regular;                    /* 38 + 1          */
       char hkey_un0;                   /* 39 + 1          */
       };                               /* total=40 bytes  */
struct image_dimension 
       {                                /* off + size      */
       short int dim[8];                /* 0 + 16          */
       short int unused8;               /* 16 + 2          */
       short int unused9;               /* 18 + 2          */
       short int unused10;              /* 20 + 2          */
       short int unused11;              /* 22 + 2          */
       short int unused12;              /* 24 + 2          */
       short int unused13;              /* 26 + 2          */
       short int unused14;              /* 28 + 2          */
       short int datatype;              /* 30 + 2          */
       short int bitpix;                /* 32 + 2          */
       short int dim_un0;               /* 34 + 2          */
       float pixdim[8];                 /* 36 + 32         */
       float funused8;                  /* 68 + 4          */
       float funused9;                  /* 72 + 4          */
       float funused10;                 /* 76 + 4          */
       float funused11;                 /* 80 + 4          */
       float funused12;                 /* 84 + 4          */
       float funused13;                 /* 88 + 4          */
       float compressed;                /* 92 + 4          */
       float verified;                  /* 96 + 4          */
       int glmax,glmin;                 /* 100 + 8         */
       };                               /* total=108 bytes */
struct data_history       
       {                                /* off + size      */
       char descrip[80];                /* 0 + 80          */
       char aux_file[24];               /* 80 + 24         */
       char orient;                     /* 104 + 1         */
       char originator[10];             /* 105 + 10        */
       char generated[10];              /* 115 + 10        */
       char scannum[10];                /* 125 + 10        */
       char patient_id[10];             /* 135 + 10        */
       char exp_date[10];               /* 145 + 10        */
       char exp_time[10];               /* 155 + 10        */
       char hist_un0[3];                /* 165 + 3         */
       int views ;                        /* 168 + 4         */
       int vols_added;                  /* 172 + 4         */
       int start_field;                 /* 176 + 4         */
       int field_skip;                  /* 180 + 4         */
       int omax, omin;                  /* 184 + 8         */
       int smax, smin;                  /* 192 + 8         */
       };
struct dsr
       { 
       struct header_key hk;            /* 0 + 40          */
       struct image_dimension dime;     /* 40 + 108        */
       struct data_history hist;        /* 148 + 200       */
       };                               /* total= 348 bytes*/   

#include "mex.h"
#include <string.h>

#define F_IN	prhs[0]
#define X_OUT   plhs[0]

#define NAME "mexreadanalyze"
#define EXTLEN 10

#define BUFLEN 1024

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{  int namelen,size,i ;
   char *name,*namel,buf[BUFLEN] ;
   FILE *f ;
   void *rdata ; /* raw data */
   struct dsr hdr ;
   int dims[10] ;
   double *ap ;
   short int *sip ;
 
   if (nrhs != 1) 
     mexErrMsgTxt(NAME " requires one input argument."); 
   if (nlhs != 1) 
     mexErrMsgTxt(NAME " requires one output argument."); 
 
   if (!mxIsChar(F_IN)) 
     mexErrMsgTxt(NAME " requires the first argument to be string."); 

   namelen = (mxGetM(F_IN) * mxGetN(F_IN)) + 1;

   name = mxCalloc(namelen, sizeof(char));
   namel = mxCalloc(namelen+EXTLEN, sizeof(char));

   if (name == NULL || namel == NULL)
     mexErrMsgTxt(NAME " Not enough heap space to hold converted string.");

   if (mxGetString(F_IN, name, namelen))
           mexErrMsgTxt(NAME " Could not convert string data.");

   strcpy(namel,name) ;
   strcat(namel,".hdr") ;
   if ((f=fopen(namel,"r"))==NULL) {
       sprintf(buf,NAME " Could not open header file `%s'.",namel) ;
       mexErrMsgTxt(buf) ;
   }
   if (fread(&hdr,sizeof(struct dsr),1,f)!=1)
       mexErrMsgTxt(NAME " Could not read from header file.") ; 
   
   mexPrintf("dim[0]=%d, dim[1]=%d, dim[2]=%d, dim[3]=%d, dim[4]=%d, type=%d, bitpix=%d\n",
	     hdr.dime.dim[0],hdr.dime.dim[1],hdr.dime.dim[2],hdr.dime.dim[3],
	     hdr.dime.dim[4],hdr.dime.datatype,hdr.dime.bitpix) ;
   fclose(f) ;

   if (hdr.dime.datatype!=4) mexErrMsgTxt(NAME " Unsupported data type.") ;

   for(size=1,i=1;i<=hdr.dime.dim[0];i++) size*=(dims[i-1]=hdr.dime.dim[i]) ;   

   strcpy(namel,name) ;
   strcat(namel,".img") ;
   if ((f=fopen(namel,"r"))==NULL) {
       sprintf(buf,NAME " Could not open image file `%s'.",namel) ;
       mexErrMsgTxt(buf) ;
   }

   rdata = mxCalloc(size*hdr.dime.bitpix/8, 1);

   if (rdata == NULL)
     mexErrMsgTxt(NAME " Not enough heap space to hold raw data.");

   if (fread(rdata,size*hdr.dime.bitpix/8,1,f)!=1)
       mexErrMsgTxt(NAME " Could not read from image file.") ; 

   fclose(f) ;

   if ((X_OUT=mxCreateNumericArray(hdr.dime.dim[0],dims,mxDOUBLE_CLASS,mxREAL))
       ==NULL) mexErrMsgTxt(NAME " Could not create array.") ;

   ap=mxGetPr(X_OUT) ;   
   mexPrintf("size=%d, dims=%d, %d %d %d %d\n",size,hdr.dime.dim[0],dims[0],dims[1],dims[2],dims[3]) ;


   for(sip=rdata,i=0;i<size;i++) 
     *ap++=(double)(*sip++) ; 

  mxFree(name) ; mxFree(namel) ; mxFree(rdata) ;
}
