/* This file writes an Analyze(TM) file */
/* for the moment, just the signed_short integer datatype */
/* J.K. 8.10.1998 */

/* info from "http://www.mayo.edu/bir/analyze/AnalyzeFileInfo.html#FileFormat" */


/* usage: mexwriteanalyze(filename,X) 
 - where filename does not contain the extension, `.hdr' and `.img' is added
automatically 
 - the multidimensional matrix is stored as signed short ints, undefined behaviour for elements out of range.
*/

#define DT_NONE                        0 
#define DT_UNKNOWN               0      /*Unknown data type*/ 
#define DT_BINARY                    1      /*Binary (1 bit per voxel)*/ 
#define DT_UNSIGNED_CHAR     2      /*Unsigned character (8 bits per voxel)*/ 
#define DT_SIGNED_SHORT        4      /*Signed short (16 bits per voxel)*/ 
#define DT_SIGNED_INT             8      /*Signed integer (32 bits per voxel)*/ 
#define DT_FLOAT                     16     /*Floating point (32 bits per voxel)*/ 
#define DT_COMPLEX                32     /*Complex (64 bits per voxel; 2 floating point numbers) 
					   #define DT_DOUBLE                   64     /*Double precision (64 bits per voxel)*/ 
#define DT_RGB                         128    /* */
#define DT_ALL                         255    /* */


struct header_key       /* header key   */ 
       {                                /* off + size      */
       int sizeof_hdr;                   /* 0 + 4           */
       char data_type[10];              /* 4 + 10          */
       char db_name[18];                /* 14 + 18         */
       int extents;                     /* 32 + 4          */
       short int session_error;         /* 36 + 2          */
       char regular;                    /* 38 + 1          */
       char hkey_un0;                   /* 39 + 1          */
       };                               /* total=40 bytes  */
struct image_dimension 
       {                                /* off + size      */
       short int dim[8];                /* 0 + 16          */
       short int unused8;               /* 16 + 2          */
       short int unused9;               /* 18 + 2          */
       short int unused10;              /* 20 + 2          */
       short int unused11;              /* 22 + 2          */
       short int unused12;              /* 24 + 2          */
       short int unused13;              /* 26 + 2          */
       short int unused14;              /* 28 + 2          */
       short int datatype;              /* 30 + 2          */
       short int bitpix;                /* 32 + 2          */
       short int dim_un0;               /* 34 + 2          */
       float pixdim[8];                 /* 36 + 32         */
       float funused8;                  /* 68 + 4          */
       float funused9;                  /* 72 + 4          */
       float funused10;                 /* 76 + 4          */
       float funused11;                 /* 80 + 4          */
       float funused12;                 /* 84 + 4          */
       float funused13;                 /* 88 + 4          */
       float compressed;                /* 92 + 4          */
       float verified;                  /* 96 + 4          */
       int glmax,glmin;                 /* 100 + 8         */
       };                               /* total=108 bytes */
struct data_history       
       {                                /* off + size      */
       char descrip[80];                /* 0 + 80          */
       char aux_file[24];               /* 80 + 24         */
       char orient;                     /* 104 + 1         */
       char originator[10];             /* 105 + 10        */
       char generated[10];              /* 115 + 10        */
       char scannum[10];                /* 125 + 10        */
       char patient_id[10];             /* 135 + 10        */
       char exp_date[10];               /* 145 + 10        */
       char exp_time[10];               /* 155 + 10        */
       char hist_un0[3];                /* 165 + 3         */
       int views ;                        /* 168 + 4         */
       int vols_added;                  /* 172 + 4         */
       int start_field;                 /* 176 + 4         */
       int field_skip;                  /* 180 + 4         */
       int omax, omin;                  /* 184 + 8         */
       int smax, smin;                  /* 192 + 8         */
       };
struct dsr
       { 
       struct header_key hk;            /* 0 + 40          */
       struct image_dimension dime;     /* 40 + 108        */
       struct data_history hist;        /* 148 + 200       */
       };                               /* total= 348 bytes*/   

#include "mex.h"
#include <string.h>

#define F_IN	prhs[0]
#define X_IN   prhs[1]

#define NAME "mexreadanalyze"
#define EXTLEN 10

#define BUFLEN 1024


void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{  int namelen,size,i ;
   char *name,*namel,buf[BUFLEN] ;
   FILE *f ;
   void *rdata ; /* raw data */
   struct dsr hdr ;
   int ndim; const int *dims ;
   double *ap ;
   short int *sip ;
 
   if (nrhs != 2) 
     mexErrMsgTxt(NAME " requires two input argument."); 
 
   if (!mxIsChar(F_IN)) 
     mexErrMsgTxt(NAME " requires the first argument to be string."); 

   namelen = (mxGetM(F_IN) * mxGetN(F_IN)) + 1;

   name = mxCalloc(namelen, sizeof(char));
   namel = mxCalloc(namelen+EXTLEN, sizeof(char));

   if (name == NULL || namel == NULL)
     mexErrMsgTxt(NAME " Not enough heap space to hold converted string.");

   if (mxGetString(F_IN, name, namelen))
           mexErrMsgTxt(NAME " Could not convert string data.");

   ndim=mxGetNumberOfDimensions(X_IN) ;
   dims=mxGetDimensions(X_IN);

   /* fill the header */
   memset(&hdr,0, sizeof(struct dsr));
   for(i=0;i<8;i++) hdr.dime.pixdim[i] = 0.0;
   
   hdr.dime.datatype = 4; /* Signed short 16bit integers */
   hdr.dime.bitpix = 16 ;

   hdr.dime.dim[0] = 4 ;  /* all Analyze images are taken as 4 dimensional */
   hdr.hk.regular = 'r';
   hdr.hk.sizeof_hdr = sizeof(struct dsr);

  for(i=1;i<=ndim;i++) hdr.dime.dim[i]=dims[i-1] ;
  for(;i<=4;i++) hdr.dime.dim[i]=1 ;

  hdr.dime.glmax  = 32767;  /* maximum voxel value  */
  hdr.dime.glmin  = -32768;  /* minimum voxel value */

/*     Set the voxel dimension fields: 
       A value of 0.0 for these fields implies that the value is unknown.
       Change these values to what is appropriate for your data
       or pass additional command line arguments     */      
    hdr.dime.pixdim[1] = 0.0; /* voxel x dimension */
    hdr.dime.pixdim[2] = 0.0; /* voxel y dimension */
    hdr.dime.pixdim[3] = 0.0; /* pixel z dimension, slice thickness */
    
/*   Planar Orientation;    */
/*   Movie flag OFF: 0 = transverse, 1 = coronal, 2 = sagittal
     Movie flag ON:  3 = transverse, 4 = coronal, 5 = sagittal  */  
    hdr.hist.orient     = 0;  

   strcpy(namel,name) ;
   strcat(namel,".hdr") ;
   if ((f=fopen(namel,"w"))==NULL) {
       sprintf(buf,NAME " Could not open header file `%s' for writing.",namel) ;
       mexErrMsgTxt(buf) ;
   }
   if (fwrite(&hdr,sizeof(struct dsr),1,f)!=1)
       mexErrMsgTxt(NAME " Could not write to header file.") ; 
   fclose(f) ;

   for(size=1,i=1;i<=hdr.dime.dim[0];i++) size*=hdr.dime.dim[i] ;   

   strcpy(namel,name) ;
   strcat(namel,".img") ;
   if ((f=fopen(namel,"w"))==NULL) {
       sprintf(buf,NAME " Could not open image file `%s' for writing.",namel) ;
       mexErrMsgTxt(buf) ;
   }

   mexPrintf("hdr.dime.dim=%d size=%d bitpix=%d\n",hdr.dime.dim[0],size,hdr.dime.bitpix) ;
   rdata = mxCalloc(size*hdr.dime.bitpix/8, 1);
   if (rdata == NULL)
     mexErrMsgTxt(NAME " Not enough heap space to hold raw data.");

   ap=mxGetPr(X_IN) ;   

   for(sip=rdata,i=0;i<size;i++) 
     *sip++=(signed int)(*ap++) ; 
   if (fwrite(rdata,size*hdr.dime.bitpix/8,1,f)!=1)
       mexErrMsgTxt(NAME " Could not write to image file.") ; 
   fclose(f) ;

  mxFree(name) ; mxFree(namel) ; mxFree(rdata) ;
}
