/* Get Wavelet Filter Coefficients
   Matlab interface to waveletfilters.c

   Usage: [g,h]=mexgetwavfilt(filt,ord) ;

  where filter: 	 			1 for "B-Spline"
	 					2 for "Cardinal Spline"
	 					3 for "Dual Spline"
	 					4 for "Orthogonal Spline"

  and order is the spline order (0=Haar,1=Linear,...).
  For synthesis, the order should be negative.

*/

#define myCreateMatrix(a,b) mxCreateDoubleMatrix(a,b,mxREAL)
#define myAlloc(a) mxMalloc(a)
#define max(a,b) ((a>b) ? (a) : (b))
#define MessageDisplay(s) mexPrintf(s) ;

#include "mex.h"
#include "waveletfilters.h"
#include "configs.h"

#define	F_IN	prhs[0]
#define	O_IN	prhs[1]
#define	G_OUT	plhs[0]
#define	H_OUT	plhs[1]

#define NAME "MEXGETWAVFILT"

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ int i,filt,ord ;
  long nh,ng; double *hp,*gp ;

  /* Check for proper number of arguments */
  
  if (nrhs != 2) {
    mexErrMsgTxt(NAME " requires two input arguments.");
  } else if (nlhs != 2) {
    mexErrMsgTxt(NAME " requires two output arguments.");
  }
  
  for (i=0;i<nrhs;i++)
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(
             NAME " needs all input arguments to be real.");
     } 
  
  filt=(int)(*(mxGetPr(F_IN))) ;
  ord=(int)(*(mxGetPr(O_IN))) ;

  if (WaveletFiltersGetSize(filt,ord,&nh,&ng)==ERROR)
    mexErrMsgTxt(NAME " WaveletFiltersGetSize failed") ;

  G_OUT = myCreateMatrix(1,ng);  gp=mxGetPr(G_OUT) ;
  H_OUT = myCreateMatrix(1,nh);  hp=mxGetPr(H_OUT) ;

  if (WaveletFiltersGetCoef(filt,ord,hp,gp)==ERROR)
    mexErrMsgTxt(NAME " WaveletFiltersGetCoef failed") ;
  
  return;
}

#include "waveletfilters.c"


