/* Wavelet analysis
   Matlab interface to wavelettools.c

   Usage: w=mexwavsplit(x,h,g,b) ;

   DWT of x using filters h,g and boundary conditions mirror (b=1) 
   or periodic (b=2). w= [lowpass highpass] 

*/

#define myCreateMatrix(a,b) mxCreateDoubleMatrix(a,b,mxREAL)
#define myAlloc(a) mxMalloc(a)
#define max(a,b) ((a>b) ? (a) : (b))
#define MessageDisplay(s) mexPrintf(s) ;

#include "mex.h"
#include "wavelettools.h"
#include "configs.h"

#define	X_IN	prhs[0]
#define	H_IN	prhs[1]
#define	G_IN	prhs[2]
#define	B_IN	prhs[3]
#define	W_OUT	plhs[0]

#define NAME "MEXWAVSPLIT"

void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{ int i,b ;
  long nh,ng,nx,nyp; double *hp,*gp,*xp,*wp ;

  /* Check for proper number of arguments */
  
  if (nrhs != 4) {
    mexErrMsgTxt(NAME " requires four input arguments.");
  } else if (nlhs != 1) {
    mexErrMsgTxt(NAME " requires one output argument.");
  }
  
  for (i=0;i<nrhs;i++)
    { if (!mxIsNumeric(prhs[i]) || mxIsComplex(prhs[i]) || 
      mxIsSparse(prhs[i])  || !mxIsDouble(prhs[i])) 
         mexErrMsgTxt(
             NAME " needs all input arguments to be real.");
     } 
  
  b=(int)(*(mxGetPr(B_IN))) ;
  hp=mxGetPr(H_IN) ; nh=mxGetM(H_IN)*mxGetN(H_IN) ;
  gp=mxGetPr(G_IN) ; ng=mxGetM(G_IN)*mxGetN(G_IN) ;
  xp=mxGetPr(X_IN) ; nx=mxGetM(X_IN)*mxGetN(X_IN) ;
  W_OUT = myCreateMatrix(1,nx);  wp=mxGetPr(W_OUT) ;

  if (WaveletSplit_1D(xp,nx,wp+nx/2,wp,&nyp,hp,nh,gp,ng,b)==ERROR)
    mexErrMsgTxt(NAME " WaveletSplit_1D failed") ;

  return;
}

#include "wavelettools.c"


