/* ------------------------------------------------------------------------- */
/* FILENAME     : HD:DEV:bilib:daniel:clib:WaveletAnalysis2D.c               */
/* AUTHOR       : Daniel Sage                                                */
/* ORGANIZATION : EPFL/DMT/IOA/BIG                                           */
/* DATE         : Mon Nov  2 15:06:51 1998                                   */
/* VERSION      :                                                            */
/* ONLINEHELP   :                                                            */
/* DEMOFILE     :                                                            */
/* DOCFILE      :                                                            */
/* URL          :                                                            */
/* ------------------------------------------------------------------------- */
/* ICI NOTE     : This file is automatically generated by ICI                */
/*                IDL-C Interface program for BILIB                          */
/*                Copyright 1998 Biomedical Imaging Group, EPFL              */
/* VERSION      : V16                                                        */
/* DATE         : 21/11/98  08:33                                            */
/* ------------------------------------------------------------------------- */


/* ------------------------------------------------------------------------- */
/* Declaration of extern procedures                                          */
/* ------------------------------------------------------------------------- */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "configs.h"
#include "debug.h"
#include "waveletanalysis2d.h"
#include "waveletanalysis.h"
#include "getput.h"
#include "messagedisplay.h"
#include "waveletfilters.h"
#include "wavelettools.h"
#include "minmax.h"

#ifdef DEBUG
#include <signal.h>
#include <errno.h>
#include <limits.h>
#include <float.h>
#endif

/* ------------------------------------------------------------------------- */
/* Declaration of static procedures                                          */
/* ------------------------------------------------------------------------- */
/* None */



/* ----------------------------------------------------------------------------
	Function:	WaveletAnalysis2DCheck
---------------------------------------------------------------------------- */
int WaveletAnalysis2DCheck( struct WaveletAnalysis2DStruct *Data) 
{
int  Status = !ERROR;
void *DataPtr = (void *)(&Data);

	DEBUG_CHECK_NULL_POINTER( WaveletAnalysis2DCheck, DataPtr, Status,
		"Unexpected error")

	if (Data->Input == (float *)NULL) {
		MessageDisplay("ERROR - Input is null");
		return(ERROR);
	}
	
	if (Data->NxInput <= 1L) {
		MessageDisplay("ERROR - NxInput should be greater than 1");
		return(ERROR);
	}
	if (Data->NyInput <= 1L) {
		MessageDisplay("ERROR - NyInput should be greater than 1");
		return(ERROR);
	}
	
	if ((Data->NxInput/2L)*2L != Data->NxInput) {  /* The input length should be even */
		MessageDisplay("ERROR - NxInput Size should be even");
		return(ERROR);
	}
	if ((Data->NyInput/2L)*2L != Data->NyInput) {  /* The input length should be even */
		MessageDisplay("ERROR - NyInput Size should be even");
		return(ERROR);
	}
	
	if (Data->Output == (float *)NULL) {
		MessageDisplay("ERROR - Output is null");
		return(ERROR);
	}
		
	if ( (strcmp(Data->Filter, "B-Spline")) &&
		 (strcmp(Data->Filter, "Cardinal Spline")) &&
		 (strcmp(Data->Filter, "Dual Spline")) &&
		 (strcmp(Data->Filter, "Orthogonal Spline")))  {
		MessageDisplay("ERROR - Filter should be equal to:");
		MessageDisplay("        B-Spline, Cardinal Spline or Orthogonal Spline");
		return(ERROR);
	}
	
	if ( (strcmp(Data->BoundaryConditions, "Periodic")) &&
		 (strcmp(Data->BoundaryConditions, "Mirror")))  {
		MessageDisplay("ERROR - BoundaryConditions should be equal to:");
		MessageDisplay("        Periodic or Mirror");
		return(ERROR);
	}
	
	if ( (strcmp(Data->Order, "0")) &&
		 (strcmp(Data->Order, "1")) &&
		 (strcmp(Data->Order, "3")) &&
		 (strcmp(Data->Order, "5")))  {
		MessageDisplay("ERROR - Order should be equal to 0, 1, 3 or 5");
		return(ERROR);
	}
	
	return(!ERROR);

}

/* ----------------------------------------------------------------------------
	Function:	WaveletAnalysis2DSize
---------------------------------------------------------------------------- */
int WaveletAnalysis2DSize( struct WaveletAnalysis2DStruct *Data) 
{
int  Status = !ERROR;
void *DataPtr = (void *)Data;

	DEBUG_CHECK_NULL_POINTER( WaveletAnalysis2DSize, DataPtr, Status,
		"Unexpected error")

	Data->NxOutput = Data->NxInput;
	Data->NyOutput = Data->NyInput;
	
	return(!ERROR);

}

/* ----------------------------------------------------------------------------
	Function:	WaveletAnalysis2D
---------------------------------------------------------------------------- */
int WaveletAnalysis2D( struct WaveletAnalysis2DStruct *Data) 
{
int  	Status = !ERROR;
void 	*DataPtr = (void *)(&Data);
short	Order, Filter, BoundaryConditions;
long	kx, ky, kz, nx, ny, nz, nmax, nout, i, j, index, jindex, nx2, ny2;
double	*FilterLowpass,	*FilterHighpass; 
long	NbLowpass[1], NbHighpass[1]; 
float	*Low, *High, *LowLow, *LowHigh, *HighLow, *HighHigh;
double	*x, *y1, *y2;

	DEBUG_CHECK_NULL_POINTER( WaveletAnalysis2D, DataPtr, Status,
		"Unexpected error")

	nx = Data->NxInput;
	ny = Data->NyInput;
	nz = 1;
	nmax = MaxLong(MaxLong(nx, ny), nz);
	
	/* Convert the Order from string to short format */
	Order = -1;
	if (!strcmp(Data->Order, "0")) Order = 0;
	if (!strcmp(Data->Order, "1")) Order = 1;
	if (!strcmp(Data->Order, "3")) Order = 3;
	if (!strcmp(Data->Order, "5")) Order = 5;
	if (Order == -1) {
		MessageDisplay("ERROR - Unexpected Order"); 
		return( ERROR);
	}

	/* Convert the Filter form string to short format */
	Filter = 0;
	if (!strcmp(Data->Filter, "B-Spline")) 			Filter = 1;
	if (!strcmp(Data->Filter, "Cardinal Spline")) 	Filter = 2;
	if (!strcmp(Data->Filter, "Dual Spline")) 		Filter = 3;
	if (!strcmp(Data->Filter, "Orthogonal Spline")) 	Filter = 4;
	if (!Filter) {
		MessageDisplay("ERROR - Unexpected Filter"); 
		return( ERROR);
	}

	/* Convert the BoundaryConditions form string to short format */
	BoundaryConditions = 0;
	if (!strcmp(Data->BoundaryConditions, "Mirror")) BoundaryConditions = 1;
	if (!strcmp(Data->BoundaryConditions, "Periodic")) BoundaryConditions = 2;
	if (!BoundaryConditions) {
		MessageDisplay("ERROR - Unexpected BoundaryConditions"); 
		return( ERROR);
	}
	
	/* Get the size of the filter */
	if ( WaveletFiltersGetSize(Filter, Order, NbLowpass, NbHighpass) == ERROR) {
		MessageDisplay("ERROR - Impossible to get the size of the filter"); 
		return( ERROR);
	}
	
	/* Allocate the memory for the highpass filter */
  	AllocateLineDouble( &FilterHighpass, *NbHighpass, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation");
		return( ERROR);
  	}
  	
	/* Allocate the memory for the lowpass filter */
  	AllocateLineDouble( &FilterLowpass, *NbLowpass, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation");
		FreeLineDouble(&FilterHighpass);
		return( ERROR);
  	}
  	
  	/* Get the coefficient of the filters */ 
	if ( WaveletFiltersGetCoef( Filter, Order, FilterLowpass, FilterHighpass) == ERROR) {
		MessageDisplay("ERROR - Impossible to get the coefficient of the filter"); 
		return( ERROR);
	}

	/* Allocate the temporary vector */ 
  	AllocateVolumeFloat( &Low, nx, ny, nz, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		return( ERROR);
  	}
  	
  	AllocateVolumeFloat( &High, nx, ny, nz, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		return( ERROR);
  	}

  	AllocateVolumeFloat( &LowLow, nx, ny, nz, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		return( ERROR);
  	}

  	AllocateVolumeFloat( &LowHigh, nx, ny, nz, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		FreeVolumeFloat(&LowLow);

		return( ERROR);
  	}

  	AllocateVolumeFloat( &HighLow, nx, ny, nz, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		FreeVolumeFloat(&LowLow);
		FreeVolumeFloat(&LowHigh);
		return( ERROR);
  	}

  	AllocateVolumeFloat( &HighHigh, nx, ny, nz, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		FreeVolumeFloat(&LowLow);
		FreeVolumeFloat(&LowHigh);
		FreeVolumeFloat(&HighLow);
		return( ERROR);
  	}

  	AllocateLineDouble( &x, nmax, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		FreeVolumeFloat(&LowLow);
		FreeVolumeFloat(&LowHigh);
		FreeVolumeFloat(&HighLow);
		FreeVolumeFloat(&HighHigh);
		return( ERROR);
  	}

  	AllocateLineDouble( &y1, nmax, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		FreeVolumeFloat(&LowLow);
		FreeVolumeFloat(&LowHigh);
		FreeVolumeFloat(&HighLow);
		FreeVolumeFloat(&HighHigh);
		FreeLineDouble(&x);
		return( ERROR);
  	}
  	
  	AllocateLineDouble( &y2, nmax, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		FreeVolumeFloat(&LowLow);
		FreeVolumeFloat(&LowHigh);
		FreeVolumeFloat(&HighLow);
		FreeVolumeFloat(&HighHigh);
		FreeLineDouble(&x);
		FreeLineDouble(&y1);
		return( ERROR);
  	}
  	
	
	/* y-processing */
	for (kz=0L;kz<nz;kz++)
	for (kx=0L;kx<nx;kx++) {
		GetyFloatToDouble(Data->Input,nx, ny, nz, kx, 0L, kz, x, ny);
		WaveletSplit_1D(	x, ny, y1, y2, &nout,
							FilterLowpass, *NbLowpass,
							FilterHighpass, *NbHighpass,
							BoundaryConditions); 
		PutyDoubleToFloat(Low, nx, ny, nz, kx, 0L, kz, y1, ny); 
		PutyDoubleToFloat(High,nx, ny, nz, kx, 0L, kz, y2, ny); 
	}
		
	/* x-processing for lowpass part*/
	for (kz=0L;kz<nz;kz++)
	for (ky=0L;ky<ny/2L;ky++) {
		GetxFloatToDouble(Low, nx, ny, nz, 0L, ky, kz, x, nx); 
		WaveletSplit_1D(	x, nx, y1, y2, &nout,
							FilterLowpass, *NbLowpass,
							FilterHighpass, *NbHighpass,
							BoundaryConditions); 
		PutxDoubleToFloat(LowLow, nx, ny, nz, 0L, ky, kz, y1, nx); 
		PutxDoubleToFloat(LowHigh,nx, ny, nz, 0L, ky, kz, y2, nx); 
	}

	/* x-processing for highpass part*/
	for (kz=0L;kz<nz;kz++)
	for (ky=0L;ky<ny/2L;ky++) {
		GetxFloatToDouble(High, nx, ny, nz, 0L, ky, kz, x, nx); 
		WaveletSplit_1D(	x, nx, y1, y2, &nout,
							FilterLowpass, *NbLowpass,
							FilterHighpass, *NbHighpass,
							BoundaryConditions); 
		PutxDoubleToFloat(HighLow, nx, ny, nz, 0L, ky, kz, y1, nx); 
		PutxDoubleToFloat(HighHigh,nx, ny, nz, 0L, ky, kz, y2, nx); 
	}
	
	/* Build the output */
	nx2 = nx / 2L;
	ny2 = ny / 2L;
	for ( i = 0L; i < nx2; i++)
	for ( j = 0L; j < ny2; j++)  {
		index  = j*nx+i;
		jindex = (j+ny2)*nx+i;
		Data->Output[index] 			= LowLow[index];
		Data->Output[index+nx2] 		= LowHigh[index];
		Data->Output[jindex] 		= HighLow[index];
		Data->Output[jindex+nx2] 	= HighHigh[index] ;
	
	}
		
	
	FreeLineDouble(&FilterHighpass);
	FreeLineDouble(&FilterLowpass);  
	FreeVolumeFloat(&Low);
	FreeVolumeFloat(&High);
	FreeVolumeFloat(&LowLow);
	FreeVolumeFloat(&LowHigh);
	FreeVolumeFloat(&HighLow);
	FreeVolumeFloat(&HighHigh);
	FreeLineDouble(&x);
	FreeLineDouble(&y1);
	FreeLineDouble(&y2);
	
	return(!ERROR);


}
