/* ------------------------------------------------------------------------- */
/* FILENAME     : HD:DEV:bilib:daniel:clib:WaveletSynthesis2D.c              */
/* AUTHOR       : Daniel Sage                                                */
/* ORGANIZATION : EPFL/DMT/IOA/BIG                                           */
/* DATE         : Mon Nov  2 15:06:51 1998                                   */
/* VERSION      :                                                            */
/* ONLINEHELP   :                                                            */
/* DEMOFILE     :                                                            */
/* DOCFILE      :                                                            */
/* URL          :                                                            */
/* ------------------------------------------------------------------------- */
/* ICI NOTE     : This file is automatically generated by ICI                */
/*                IDL-C Interface program for BILIB                          */
/*                Copyright 1998 Biomedical Imaging Group, EPFL              */
/* VERSION      : V16                                                        */
/* DATE         : 21/11/98  09:53                                            */
/* ------------------------------------------------------------------------- */


/* ------------------------------------------------------------------------- */
/* Declaration of extern procedures                                          */
/* ------------------------------------------------------------------------- */

#include <stdlib.h>
#include <string.h>
#include "configs.h"
#include "debug.h"
#include "waveletsynthesis2d.h"
#include "waveletsynthesis.h"
#include "getput.h"
#include "messagedisplay.h"
#include "waveletfilters.h"
#include "wavelettools.h"
#include "minmax.h"

#ifdef DEBUG
#include <signal.h>
#include <errno.h>
#include <limits.h>
#include <float.h>
#endif

/* ------------------------------------------------------------------------- */
/* Declaration of static procedures                                          */
/* ------------------------------------------------------------------------- */
/* None */


/* ----------------------------------------------------------------------------
	Function:	WaveletSynthesis2DCheck
---------------------------------------------------------------------------- */
int WaveletSynthesis2DCheck( struct WaveletSynthesis2DStruct *Data) 
{
int  Status = !ERROR;
void *DataPtr = (void *)(&Data);

	DEBUG_CHECK_NULL_POINTER( WaveletSynthesis2DCheck, DataPtr, Status,
		"Unexpected error")
	if (Data->Input == (float *)NULL) {
		MessageDisplay("ERROR - Input is null");
		return(ERROR);
	}
	
	if (Data->NxInput <= 1) {
		MessageDisplay("ERROR - NxInput should be greater than 1");
		return(ERROR);
	}
	if (Data->NyInput <= 1) {
		MessageDisplay("ERROR - NyInput should be greater than 1");
		return(ERROR);
	}
		
	if (Data->Output == (float *)NULL) {
		MessageDisplay("ERROR - Output is null");
		return(ERROR);
	}
		
	if ( (strcmp(Data->Filter, "B-Spline")) &&
		 (strcmp(Data->Filter, "Cardinal Spline")) &&
		 (strcmp(Data->Filter, "Dual Spline")) &&
		 (strcmp(Data->Filter, "Orthogonal Spline")))  {
		MessageDisplay("ERROR - Filter should be equal to:");
		MessageDisplay("        B-Spline, Cardinal Spline or Orthogonal Spline");
		return(ERROR);
	}
	
	if ( (strcmp(Data->BoundaryConditions, "Periodic")) &&
		 (strcmp(Data->BoundaryConditions, "Mirror")))  {
		MessageDisplay("ERROR - BoundaryConditions should be equal to:");
		MessageDisplay("        Periodic or Mirror");
		return(ERROR);
	}
	
	if ( (strcmp(Data->Order, "0")) &&
		 (strcmp(Data->Order, "1")) &&
		 (strcmp(Data->Order, "3")) &&
		 (strcmp(Data->Order, "5")))  {
		MessageDisplay("ERROR - Order should be equal to 0, 1, 3 or 5");
		return(ERROR);
	}

	return(!ERROR);

}

/* ----------------------------------------------------------------------------
	Function:	WaveletSynthesis2DSize
---------------------------------------------------------------------------- */
int WaveletSynthesis2DSize( struct WaveletSynthesis2DStruct *Data) 
{
int  Status = !ERROR;
void *DataPtr = (void *)Data;

	DEBUG_CHECK_NULL_POINTER( WaveletSynthesis2DSize, DataPtr, Status,
		"Unexpected error")

	Data->NxOutput = Data->NxInput;
	Data->NyOutput = Data->NyInput;

	return(!ERROR);

}

/* ----------------------------------------------------------------------------
	Function:	WaveletSynthesis2D
---------------------------------------------------------------------------- */
int WaveletSynthesis2D( struct WaveletSynthesis2DStruct *Data) 
{
int  	Status = !ERROR;
void 	*DataPtr = (void *)(&Data);
short	Order, Filter, BoundaryConditions;
long	kx, ky, kz, nx, ny, nz, nmax, nout, i, j, index, nx2, ny2;
double	*FilterLowpass,	*FilterHighpass; 
long	NbLowpass[1], NbHighpass[1]; 
float	*Low, *High, *LowLow, *LowHigh, *HighLow, *HighHigh;
double	*x, *y1, *y2;

	DEBUG_CHECK_NULL_POINTER( WaveletSynthesis2D, DataPtr, Status,
		"Unexpected error")

	nx = Data->NxInput;
	ny = Data->NyInput;
	nz = 1;
	nmax = MaxLong(MaxLong(nx, ny), nz);
	
	/* Convert the Order from string to short format */
	Order = -1;
	if (!strcmp(Data->Order, "0")) Order = 0;
	if (!strcmp(Data->Order, "1")) Order = 1;
	if (!strcmp(Data->Order, "3")) Order = 3;
	if (!strcmp(Data->Order, "5")) Order = 5;
	if (Order == -1) {
		MessageDisplay("ERROR - Unexpected Order"); 
		return( ERROR);
	}

	/* Convert the Filter form string to short format */
	Filter = 0;
	if (!strcmp(Data->Filter, "B-Spline")) 			Filter = 1;
	if (!strcmp(Data->Filter, "Cardinal Spline")) 	Filter = 2;
	if (!strcmp(Data->Filter, "Dual Spline")) 		Filter = 3;
	if (!strcmp(Data->Filter, "Orthogonal Spline")) 	Filter = 4;
	if (!Filter) {
		MessageDisplay("ERROR - Unexpected Filter"); 
		return( ERROR);
	}

	/* Convert the BoundaryConditions form string to short format */
	BoundaryConditions = 0;
	if (!strcmp(Data->BoundaryConditions, "Mirror")) BoundaryConditions = 1;
	if (!strcmp(Data->BoundaryConditions, "Periodic")) BoundaryConditions = 2;
	if (!BoundaryConditions) {
		MessageDisplay("ERROR - Unexpected BoundaryConditions"); 
		return( ERROR);
	}

	/* Get the size of the filter */
	if ( WaveletFiltersGetSize(Filter, (short)(-Order), NbLowpass, NbHighpass) == ERROR) {
		MessageDisplay("ERROR - Impossible to get the size of the filter"); 
		return( ERROR);
	}
		
	/* Allocate the memory for the highpass filter */
  	AllocateLineDouble( &FilterHighpass, *NbHighpass, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation (FilterHighpass)");
		return( ERROR);
  	}
  	
	/* Allocate the memory for the lowpass filter */
  	AllocateLineDouble( &FilterLowpass, *NbLowpass, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation (FilterLowpass)");
		FreeLineDouble(&FilterHighpass);
		return( ERROR);
  	}
  	
  	/* Get the coefficient of the filters */ 
	if ( WaveletFiltersGetCoef( Filter, (short)(-Order), FilterLowpass, FilterHighpass) == ERROR) {
		MessageDisplay("ERROR - Impossible to get the coefficient of the filter"); 
		return( ERROR);
	}
	
	/* Allocate the temporary vector */ 
  	AllocateVolumeFloat( &Low, nx, ny, nz, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation (Low)");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		return( ERROR);
  	}
  	
  	AllocateVolumeFloat( &High, nx, ny, nz, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation (High)");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		return( ERROR);
  	}

  	AllocateVolumeFloat( &LowLow, nx, ny, nz, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation (LowLow)");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		return( ERROR);
  	}

  	AllocateVolumeFloat( &LowHigh, nx, ny, nz, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation (LowHigh)");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		FreeVolumeFloat(&LowLow);

		return( ERROR);
  	}

  	AllocateVolumeFloat( &HighLow, nx, ny, nz, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation (HighLow)");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		FreeVolumeFloat(&LowLow);
		FreeVolumeFloat(&LowHigh);
		return( ERROR);
  	}

  	AllocateVolumeFloat( &HighHigh, nx, ny, nz, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation (HighHigh)");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		FreeVolumeFloat(&LowLow);
		FreeVolumeFloat(&LowHigh);
		FreeVolumeFloat(&HighLow);
		return( ERROR);
  	}

  	AllocateLineDouble( &x, nmax, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation (x)");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		FreeVolumeFloat(&LowLow);
		FreeVolumeFloat(&LowHigh);
		FreeVolumeFloat(&HighLow);
		FreeVolumeFloat(&HighHigh);
		return( ERROR);
  	}

  	AllocateLineDouble( &y1, nmax, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation (y1)");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		FreeVolumeFloat(&LowLow);
		FreeVolumeFloat(&LowHigh);
		FreeVolumeFloat(&HighLow);
		FreeVolumeFloat(&HighHigh);
		FreeLineDouble(&x);
		return( ERROR);
  	}
  	
  	AllocateLineDouble( &y2, nmax, &Status);
  	if (Status == ERROR) {
		MessageDisplay( "ERROR - Unable to perform allocation (y2)");
		FreeLineDouble(&FilterHighpass);
		FreeLineDouble(&FilterLowpass);  
		FreeVolumeFloat(&Low);
		FreeVolumeFloat(&High);
		FreeVolumeFloat(&LowLow);
		FreeVolumeFloat(&LowHigh);
		FreeVolumeFloat(&HighLow);
		FreeVolumeFloat(&HighHigh);
		FreeLineDouble(&x);
		FreeLineDouble(&y1);
		return( ERROR);
  	}
  	
	
	/* Extract component from the input */
	nx2 = MaxLong(nx / 2L, 1L);
	ny2 = MaxLong(ny / 2L, 1L);
	for ( i = 0L; i < nx2; i++)
	for ( j = 0L; j < ny2; j++)  {
		index = j*nx+i;
		LowLow[index] 	= Data->Input[index];
		LowHigh[index] 	= Data->Input[index+nx2];
		HighLow[index] 	= Data->Input[(j+ny2)*nx+i];
		HighHigh[index] = Data->Input[i+nx2+(j+ny2)*nx];
	
	}
	
	/* x-processing for lowpass part*/
	for (kz=0L;kz<nz;kz++)
	for (ky=0L;ky<ny/2L;ky++) {
		GetxFloatToDouble(LowLow,  nx, ny, nz, 0L, ky, kz, y1, nx); 
		GetxFloatToDouble(LowHigh, nx, ny, nz, 0L, ky, kz, y2, nx); 
		WaveletMerge_1D(	y1, y2, nx2, x, &nout,
							FilterLowpass, *NbLowpass,
							FilterHighpass, *NbHighpass,
							BoundaryConditions); 
		PutxDoubleToFloat(Low, nx, ny, nz, 0L, ky, kz, x, nx); 
	}

	/* x-processing for highpass part*/
	for (kz=0L;kz<nz;kz++)
	for (ky=0L;ky<ny/2L;ky++) {
		GetxFloatToDouble(HighLow,  nx, ny, nz, 0L, ky, kz, y1, nx); 
		GetxFloatToDouble(HighHigh, nx, ny, nz, 0L, ky, kz, y2, nx); 
		WaveletMerge_1D(	y1, y2, nx2, x, &nout,
							FilterLowpass, *NbLowpass,
							FilterHighpass, *NbHighpass,
							BoundaryConditions); 
		PutxDoubleToFloat(High, nx, ny, nz, 0L, ky, kz, x, nx); 
	}
		
	/* y-processing */
	for (kz=0L;kz<nz;kz++)
	for (kx=0L;kx<nx;kx++) {
		GetyFloatToDouble(Low,  nx, ny, nz, kx, 0L, kz, y1, ny);
		GetyFloatToDouble(High, nx, ny, nz, kx, 0L, kz, y2, ny);
		WaveletMerge_1D(	y1, y2, ny2, x, &nout,
							FilterLowpass, *NbLowpass,
							FilterHighpass, *NbHighpass,
							BoundaryConditions); 
		PutyDoubleToFloat(Data->Output,nx, ny, nz, kx, 0L, kz, x, ny); 
	}

	
	/* --- Exit --- */
	FreeLineDouble(&FilterHighpass);
	FreeLineDouble(&FilterLowpass);  
	FreeVolumeFloat(&Low);
	FreeVolumeFloat(&High);
	FreeVolumeFloat(&LowLow);
	FreeVolumeFloat(&LowHigh);
	FreeVolumeFloat(&HighLow);
	FreeVolumeFloat(&HighHigh);
	FreeLineDouble(&x);
	FreeLineDouble(&y1);
	FreeLineDouble(&y2);
	
	
	return(!ERROR);

}
